@@ -56,6 +56,8 @@
 #undef CFCRAY
 #undef CFGNU
 #undef UNIXWARE
+#undef HAVE_SYS_ACL_H
+#undef NOTBROKEN
 
 /* SVR4 header stuff */
 
