@@ -446,7 +446,9 @@ enum actions
    unmounta,
    admit,
    deny,
-   acls
+   acls,
+   interfaces,
+   plugin
    };
 
 /*******************************************************************/
@@ -536,6 +538,10 @@ enum fileattr
    cfacl,
    cfpurge,
    cfuseshell,
+   cfsetlog,
+   cfsetinform,
+   cfsetnetmask,
+   cfsetbroadcast,
    cfbad
    };
 
@@ -560,6 +566,7 @@ enum vnames
    cfsite,
    cfhost,
    cffqhost,
+   cfipaddr,
    cfbinserver,
    cfsysadm,
    cfdomain,
@@ -573,6 +580,7 @@ enum vnames
    cfmountpat,
    cfhomepat,
    cfaddclass,
+   cfinstallclass,
    cfaccess,
    cfclass,
    cfarch,
@@ -726,7 +734,10 @@ enum editnames
    AppendToLineIfNotContains,
    DeleteLinesAfterThisMatching,
    DefineClasses,
-   CatchAbort
+   CatchAbort,
+   EditBackup,
+   EditLog,
+   EditInform
    };
 
 enum editnames EditActionsToCode();
@@ -812,6 +823,17 @@ char *CompressedArrayValue();
 
 /*******************************************************************/
 
+struct Interface
+   {
+   char *ifdev;
+   char *netmask;
+   char *broadcast;
+   char *classes;
+   struct Interface *next;
+   };
+
+/*******************************************************************/
+
 struct Item
    {
    char *name;
@@ -852,6 +874,10 @@ struct Process
    char           action;
    char           *classes;
    char           useshell;
+   char           log;
+   char           inform;
+   struct Item    *exclusions;
+   struct Item    *inclusions;      
    struct Process *next;
    };
 
@@ -887,6 +913,8 @@ struct Tidy
          char               dirlinks;          /* k=keep, t=tidy */
          char               rmdirs;            /* t=true, f=false */
          char               searchtype;        /* a, m, c time */
+	 char               log;
+	 char               inform;
          struct TidyPattern *next;
          };
 
@@ -930,6 +958,8 @@ struct File
    struct GidList *gid;
    struct File *next;
    struct Item *acl_aliases;
+   char   log;
+   char   inform;
    };
 
 /*******************************************************************/
@@ -941,6 +971,8 @@ struct Disk
    char  *define;
    int    freespace;
    struct Disk *next;
+   char   log;
+   char   inform;
    };
 
 /*******************************************************************/
@@ -955,6 +987,8 @@ struct Disable
    char   comp;
    char   *defines;
    struct Disable *next;
+   char   log;
+   char   inform;
    };
 
 /*******************************************************************/
@@ -991,6 +1025,8 @@ struct Image
    
    struct Image *next;
    struct Item *acl_aliases;
+   char   log;
+   char   inform;
    };
 
 /*******************************************************************/
@@ -1066,6 +1102,8 @@ struct Link
    struct Item *inclusions;
    struct Item *copy;
    struct Link *next;
+   char   log;
+   char   inform;
    };
 
 /*******************************************************************/
@@ -1098,6 +1136,8 @@ struct ShellComm
    int               timeout;
    char              useshell;
    struct ShellComm  *next;
+   char              log;
+   char              inform;
    };
 
 /*******************************************************************/
@@ -1146,7 +1186,7 @@ struct Auth
 # define S_ISDIR(m)      (((m) & S_IFMT) == S_IFDIR)
 #endif
 #ifndef S_ISLNK
-# define S_ISLNK(m)      (((m)&S_IFMT) == S_IFLNK)
+# define S_ISLNK(m)      (((m) & S_IFMT) == S_IFLNK)
 #endif
 #ifndef S_ISFIFO
 # define S_ISFIFO(m)     (((m) & S_IFMT) == S_IFIFO)
