@@ -1,6 +1,6 @@
 /* 
 
-        Copyright (C) 1995,96,97
+        Copyright (C) 1995,96,97,98
         Free Software Foundation, Inc.
 
    This file is part of GNU cfengine - written and maintained 
@@ -28,8 +28,9 @@
 /*  Cfengine : remote server daemon example                        */
 /*                                                                 */
 /*  This uses heavy weight processes, which is not very efficient  */
+/*  or pthreads if you have a working implementation               */
 /*                                                                 */
-/*  Mark Burgess 1997                                              */
+/*  Mark Burgess 1997,8                                            */
 /*                                                                 */
 /*******************************************************************/
 
@@ -45,7 +46,7 @@
 # include <tcpd.h>
 #endif
 
-#ifdef HAVE_DB_H
+#if defined HAVE_DB_H && defined HAVE_DB_185_H
 # include <db.h>
 #endif
 
@@ -62,8 +63,7 @@
 #endif
 
 #ifdef HAVE_PTHREAD_H
-# define MAXTHREADS 10
-pthread_t THREADS[MAXTHREADS];
+pthread_attr_t PTHREADDEFAULTS;
 #endif
 
 /*******************************************************************/
@@ -116,10 +116,13 @@ InitHashTable();
 
 AddClassToHeap("any");      /* This is a reserved word / wildcard */
 
-while ((c=getopt_long(argc,argv,"d:f:vhpFV",CFDOPTIONS,&optindex)) != EOF)
+while ((c=getopt_long(argc,argv,"d:f:vhpFVm",CFDOPTIONS,&optindex)) != EOF)
   {
   switch ((char) c)
       {
+      case 'm': MULTITHREAD = true;
+                break;
+
       case 'f': strcpy(VINPUTFILE,optarg);
                 break;
 
@@ -176,7 +179,6 @@ strcpy(VLOGDIR,LOGFILEDIR);
 strcpy(VDOMAIN,"undefined.domain");
 
 strcpy(VCANONICALFILE,CanonifyName(VINPUTFILE));
- 
 }
 
 /*******************************************************************/
@@ -186,6 +188,19 @@ CheckVariables()
 { struct stat statbuf;
   int i, value = -1;
 
+#if defined HAVE_LIBPTHREAD && defined NOTBROKEN
+
+if (MULTITHREAD)
+   {
+   CfLog(cfinform,"Multithreaded version","");
+   }
+
+#else
+ 
+ CfLog(cfinform,"Single threaded version","");
+
+#endif
+  
 if ((CFDSTARTTIME = time((time_t *)NULL)) == -1)
    {
    printf("Couldn't read system clock\n");
@@ -246,24 +261,46 @@ if (GetMacroValue("MaxConnections"))
    {
    bzero(VBUFF,bufsize);
    ExpandVarstring("$(MaxConnections)",VBUFF,NULL);
+   Debug("$(MaxConnections) Expanded to %s\n",VBUFF);
+
 
    CFD_MAXPROCESSES = atoi(VBUFF);
 
-   if (CFD_MAXPROCESSES < 1)
+   if ((CFD_MAXPROCESSES < 1) || (CFD_MAXPROCESSES > 32))
       {
       FatalError("cfd: MaxConnections with silly value");
       }
-
-   Debug("MaxConnections = %d\n",CFD_MAXPROCESSES);
-   }
+    }
 else
    {
    CFD_MAXPROCESSES = 10;
    }
 
+if (GetMacroValue("AutoExecInterval"))
+   {
+   bzero(VBUFF,bufsize);
+   ExpandVarstring("$(AutoExecInterval)",VBUFF,NULL);
+   Debug("$(AutoExecInterval) Expanded to %s\n",VBUFF);
+
+   CFD_INTERVAL = 60 * atoi(VBUFF);
+
+   if ((CFD_INTERVAL < 0) || (CFD_INTERVAL > 60*24*7))
+      {
+      FatalError("cfd: AutoExecInterval with silly value in minutes (>0 and less than 1 week)");
+      }
+   }
+else
+   {
+   CFD_INTERVAL = 0;
+   }
+
+ Debug("MaxConnections = %d\n",CFD_MAXPROCESSES);
+ Debug("AutoExecInterval = %d\n",CFD_INTERVAL);
+
+ 
 i = 0;
 
-if (strstr(VSYSNAME.nodename,VDOMAIN))
+if (strstr(VSYSNAME.nodename,ToLowerStr(VDOMAIN)))
    {
    strcpy(VFQNAME,VSYSNAME.nodename);
    
@@ -275,7 +312,7 @@ if (strstr(VSYSNAME.nodename,VDOMAIN))
    }
 else
    {
-   sprintf(VFQNAME,"%s.%s",VSYSNAME.nodename,VDOMAIN);
+   sprintf(VFQNAME,"%s.%s",VSYSNAME.nodename,ToLowerStr(VDOMAIN));
    strcpy(VUQNAME,VSYSNAME.nodename);
    }
 }
@@ -333,10 +370,12 @@ char **argv;
 { struct sockaddr_in cin, sin; 
   char recvbuffer[bufsize], sendbuffer[bufsize];
   struct servent *server;
-  int sd, addrlen = sizeof(cin);
-  struct cfd_connection conn;
+  int sd, sd_reply, addrlen = sizeof(cin);
+  struct cfd_connection *conn;
   int portnumber, yes=1, shmid;
-  void ExitCleanly();
+  struct cfd_connection *NewConn();
+  struct linger cflinger = { 1,60 };
+  void ExitCleanly(), AutoExec();
 
 
 openlog(VPREFIX,LOG_PID|LOG_NOWAIT|LOG_ODELAY,LOG_DAEMON);
@@ -363,23 +402,19 @@ signal(SIGINT,ExitCleanly);
 signal(SIGTERM,ExitCleanly);
 signal(SIGHUP,SIG_IGN);
 signal(SIGPIPE,SIG_IGN);
+signal(SIGALRM,(void *)AutoExec);
 
-#ifdef HAVE_PTHREAD_T 
+sprintf(OUTPUT,"Setting alarm to %d seconds\n",CFD_INTERVAL); 
+CfLog(cfinform,OUTPUT,""); 
+alarm(CFD_INTERVAL);
 
-for (i = 0; i < MAXTHREADS; i++)
-   {
-   THREADS[i] = NULL;
-   }
- 
-#endif
- 
 if ((server = getservbyname(CFENGINE_SERVICE,"tcp")) == NULL)
    {
    CfLog(cferror,"Couldn't get cfengine service","getservbyname");
    exit (1);
    }
 
- bzero(&cin,sizeof(cin));
+bzero(&cin,sizeof(cin));
 
 sin.sin_port = (unsigned short)(server->s_port); /*  Service returns network byte order */
 sin.sin_addr.s_addr = INADDR_ANY;
@@ -397,6 +432,13 @@ if (setsockopt (sd, SOL_SOCKET, SO_REUSEADDR, (char *) &yes, sizeof (int)) == -1
    exit (1);
    }
 
+ if (setsockopt (sd, SOL_SOCKET, SO_LINGER, (char *) &cflinger, sizeof (struct linger)) == -1)
+   {
+   CfLog(cferror,"Couldn't set socket options","sockopt");
+   exit (1);
+   }
+
+
 if (bind(sd,(struct sockaddr *)&sin,sizeof(sin)) == -1)  /* Must have this on server */
    {
    CfLog(cferror,"Couldn't bind to socket","bind");
@@ -413,20 +455,19 @@ Verbose("Listening for connections on port %d..\n",server->s_port);
 
 while (true)
    {
-   if ((conn.sd_reply = accept(sd, (struct sockaddr *)&cin, &addrlen)) == -1)
+   if ((sd_reply = accept(sd, (struct sockaddr *)&cin, &addrlen)) == -1)
       {
       continue;
       }
    else
       {
       Debug("New connection...\n");
-      conn.sd_recv = sd;
+      conn = NewConn(sd_reply);
 
-      while (!FinishedWithConnection(recvbuffer,sendbuffer,&conn))
+      while (BusyWithConnection(recvbuffer,sendbuffer,conn))
          {
          }
 
-      close(conn.sd_reply);
       Debug("End of session\n");
 
       CheckFileChanges(argc,argv,sd);
@@ -458,7 +499,7 @@ printf("Info & fixes at http://www.iu.hioslo.no/~mark/cfengine.html\n");
 
 /*********************************************************************/
 
-FinishedWithConnection(recvbuffer,sendbuffer,conn)
+BusyWithConnection(recvbuffer,sendbuffer,conn)
 
 char recvbuffer[], sendbuffer[];
 struct cfd_connection *conn;
@@ -470,11 +511,11 @@ struct cfd_connection *conn;
 { time_t tloc, trem = 0;
   char filename[bufsize],args[bufsize];
   long time_no_see = 0;
-  struct cfd_thread_arg thr_args;
+  struct cfd_thread_arg *thr_args;
 
 if (RecvSocketStream(conn->sd_reply,recvbuffer,bufsize,0) == -1)
    {
-   return true;
+   return false;
    }
 
 if (strlen(recvbuffer) == 0)
@@ -492,51 +533,67 @@ switch (GetCommand(recvbuffer))
 		     if (! conn->id_verified)
 			{
 			RefuseAccess(conn->sd_reply,sendbuffer);
-			return true;
+			DeleteConn(conn);
+			return false;
 			}
 
 		     if (!AccessControl(CFRUNCOMMAND,conn->hostname))
 			{
 			RefuseAccess(conn->sd_reply,sendbuffer);
-			return true;			
+			DeleteConn(conn);
+			return false;			
 			}
 
      		     if (!MatchClasses(conn,recvbuffer))
 			{
 			Terminate(conn->sd_reply);
-			return true;
+			DeleteConn(conn);
+			return false;
 			}
 
+		     alarm(CFD_INTERVAL); /* reset alarm */
+
 		     DoExec(conn,sendbuffer,args);
 		     Terminate(conn->sd_reply);
-		     return true;
+		     DeleteConn(conn);
+		     return false;
 		     
    case cfd_auth:    conn->id_verified = VerifyConnection(conn,recvbuffer+strlen("AUTH "));
-                     return false; /* not finished yet */
+                     return true; /* not finished yet */
 		     
    case cfd_get:     bzero(filename,bufsize);
                      sscanf(recvbuffer,"GET %s",filename);
 
-		     if (! conn->id_verified)
+ 	             if (! conn->id_verified)
 			{
 			RefuseAccess(conn->sd_reply,sendbuffer);
-			return true;
+			DeleteConn(conn);
+			return false;
 			}
 
 		     if (!AccessControl(filename,conn->hostname))
 			{
 			RefuseAccess(conn->sd_reply,sendbuffer);
-			return true;			
+			DeleteConn(conn);
+			return false;			
 			}
 
 		     bzero(sendbuffer,bufsize);
-
-		     thr_args.sd_reply = conn->sd_reply;
-		     thr_args.replybuff = sendbuffer;
-		     thr_args.replyfile = filename;
 		     
-		     SpawnCfGetFile(&thr_args);
-		     return true;
+		     if ((thr_args = malloc(sizeof(struct cfd_thread_arg))) == NULL)
+			{
+			CfLog(cferror,"Could not allocate memory while serving file","malloc");
+			return false;
+			}
+		     
+		     thr_args->connect = conn;
+		     thr_args->replybuff = sendbuffer;
+		     thr_args->replyfile = strdup(filename); /* private to thread */
+		     
+		     SpawnCfGetFile(thr_args);
+		     
+		     /* close in spawn */
+		     return false;
                      break;
 		     
    case cfd_opendir: bzero(filename,bufsize);
@@ -545,23 +602,27 @@ switch (GetCommand(recvbuffer))
 		     if (! conn->id_verified)
 			{
 			RefuseAccess(conn->sd_reply,sendbuffer);
-			return true;
+			DeleteConn(conn);
+			return false;
 			}
 
 		     if (!AccessControl(filename,conn->hostname))
 			{
 			RefuseAccess(conn->sd_reply,sendbuffer);
-			return true;			
+			DeleteConn(conn);
+			return false;			
 			}		     
 
 		     CfOpenDirectory(conn,sendbuffer,filename);
-                     return true;
+		     DeleteConn(conn);
+                     return false;
 		     
    case cfd_synch:
 		     if (! conn->id_verified)
 			{
 			RefuseAccess(conn->sd_reply,sendbuffer);
-			return true;
+			DeleteConn(conn);
+			return false;
 			}
 		     
                      bzero(filename,bufsize);
@@ -576,7 +637,8 @@ switch (GetCommand(recvbuffer))
 		     
 		     if ((tloc = time((time_t *)NULL)) == -1)
                         {
-                        printf("Couldn't read system clock\n");
+                        sprintf(OUTPUT,"Couldn't read system clock\n");
+			CfLog(cfinform,OUTPUT,"time");
                         }
 
                      if (tloc - trem > CLOCK_DRIFT)
@@ -586,26 +648,28 @@ switch (GetCommand(recvbuffer))
 			}
 		     else
 			{
-			Verbose("Clocks were off by %ld\n",(long)tloc-(long)trem);
-			Verbose("Statting file %s\n",filename);
+			sprintf(OUTPUT,"Clocks were off by %ld\n",(long)tloc-(long)trem);
+			CfLog(cfverbose,OUTPUT,"");
 
 			StatFile(conn->sd_reply,sendbuffer,filename);
 			}
-		     
-		     return true;
+
+		     DeleteConn(conn);
+		     return false;
 
    case cfd_md5:
 		     if (! conn->id_verified)
 			{
 			RefuseAccess(conn->sd_reply,sendbuffer);
-			return true;
+			return false;
 			}
 		     
                      bzero(filename,bufsize);
 		     bzero(args,bufsize);
 		     
                      CompareLocalChecksum(conn,sendbuffer,recvbuffer);
-		     return true;		     
+		     DeleteConn(conn);
+		     return false;		     
    }
 
 sprintf (sendbuffer,"BAD: Malformed protocol request\n%c",EOF);
@@ -618,7 +682,8 @@ if (send(conn->sd_reply,sendbuffer,strlen(sendbuffer)+1,0) == -1)
    }
 
 Verbose("Bad command\n");
-return true;
+DeleteConn(conn);
+return false;
 }
 
 /**************************************************************/
@@ -686,6 +751,7 @@ if (CFDSTARTTIME < newstat.st_ctime)
    GetNameInfo();
    ParseInputFiles();
    CheckVariables();
+   alarm(CFD_INTERVAL);
    }
 }
 
@@ -699,7 +765,8 @@ sprintf(OUTPUT,"Exit -- (waiting for threads), pid = %d\n",getpid());
 CfLog(cfinform,OUTPUT,"");
 closelog();
 
-#ifdef HAVE_PTHREAD_H 
+#ifdef HAVE_PTHREAD_H
+/* Do we care about this? 
 for (i = 0; i < MAXTHREADS; i++)
    {
    if (THREADS[i] != NULL)
@@ -707,11 +774,104 @@ for (i = 0; i < MAXTHREADS; i++)
       pthread_join(THREADS[i],(void **)NULL);
       }
    }
+   */
 #endif 
  
 exit(0);
 }
 
+/**************************************************************/
+
+void AutoExec()
+
+{ char logbuffer[bufsize], line[bufsize], *sp;
+  FILE *pp; 
+  int print;
+  
+alarm(0);
+VBUFF[0] = '\0';
+ 
+ExpandVarstring("$(AutoExecCommand)",VBUFF,"");
+
+if (strlen(VBUFF) == 0)
+   {
+   CfLog(cferror,"No $(AutoExecCommand) variable defined","");
+   signal(SIGALRM,(void *)AutoExec);
+   return;
+   }
+
+/* Use same lock as for cfrun */
+ 
+ if (!GetLock("cfd","exec",VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
+   {
+   sprintf(OUTPUT,"cfd: Couldn't get a lock -- too soon: IfElapsed %d, ExpireAfter %d\n",VIFELAPSED,VEXPIREAFTER);
+   CfLog(cferror,OUTPUT,"");
+   
+   return;
+   }
+ 
+if (fork() == 0)
+   {
+   signal(SIGPIPE,SIG_IGN);
+   
+   ExpandVarstring("$(AutoExecCommand) --no-splay --inform",VBUFF,"");
+
+   sprintf(OUTPUT,"Interval experied, executing command %s\n",VBUFF);
+   CfLog(cfinform,OUTPUT,""); 
+
+   bzero(logbuffer,bufsize);
+
+   if ((pp = cfpopen(VBUFF,"r")) == NULL)
+      {
+      sprintf(OUTPUT,"Couldn't open pipe to command %s\n",VBUFF);
+      CfLog(cferror,OUTPUT,"cfpopen");
+
+      return;
+      }
+   
+   while (!feof(pp))
+      {
+      if (ferror(pp))  /* abortable */
+	 {
+	 fflush(pp);
+	 break;
+	 }
+      
+      ReadLine(line,1,pp);
+      
+      if (ferror(pp))  /* abortable */
+	 {
+	 fflush(pp);
+	 break;
+	 }	 
+      
+      print = false;
+      
+      for (sp = line; *sp != '\0'; sp++)
+	 {
+	 if (! isspace(*sp))
+	    {
+	    print = true;
+	    break;
+	    }
+	 }
+      
+      if (print)
+	 {
+	 sprintf(logbuffer,"%s\n",line);
+	 CfLog(cfinform,logbuffer,"");
+	 }
+      }
+   
+   cfpclose(pp);
+   closelog();
+   exit(0);
+   }
+
+signal(SIGALRM,(void *)AutoExec);
+alarm(CFD_INTERVAL); 
+}
+
 /**************************************************************/
 /* Level 3                                                    */
 /**************************************************************/
@@ -760,6 +920,7 @@ while (true)
       if (IsDefinedClass(ip->name))
 	 {
 	 Debug("Class %s matched, accepting...\n",ip->name);
+	 DeleteItemList(classlist);
 	 return true;
 	 }
       
@@ -994,7 +1155,7 @@ strcpy(name2,ToLowerStr(inet_ntoa(saddr.sin_addr)));
 strcpy(name3,ToLowerStr(ipstring));
  
 sprintf(OUTPUT,"Socket originates from %s=%s\n",name2,name1);
-CfLog(cfinform,OUTPUT,""); 
+CfLog(cfverbose,OUTPUT,""); 
 
 #ifdef HAVE_LIBWRAP
 
@@ -1077,16 +1238,9 @@ found = false;
     return false;
     } 
  
-conn->hostname = strdup(name1);
-
-if (conn->hostname != NULL)
-   {
-   Debug("Host ID verified as %s\n",name1);
-   strcpy(conn->hostname,name1);
-   return true;   
-   }
-
-return false;
+Debug("Host ID verified as %s\n",name1);
+strncpy(conn->hostname,name1,maxvarsize);
+return true;   
 }
 
 /**************************************************************/
@@ -1097,7 +1251,7 @@ int sd;
 char *sendbuffer;
 
 {
-sprintf(sendbuffer,"BAD: Host authentication failed - did you set the correct domainname?\n");
+sprintf(sendbuffer,"BAD: authentication failed - did you set the correct permissions and domainname in cfd.conf?\n");
 CfLog(cfinform,"Host authentication failed\n","");
 
 if (send(sd,sendbuffer,bufsize,0) == -1)
@@ -1178,72 +1332,56 @@ SpawnCfGetFile(args)
 
 struct cfd_thread_arg *args;
 
-{ int tid, count = 0;
+{
+#ifdef HAVE_PTHREAD_H
+ pthread_t tid;
+#endif
   void *CfGetFile();
   
 #if defined HAVE_LIBPTHREAD && defined NOTBROKEN
-/*  
-  pthread_mutex_t mutex;
 
-if (pthread_mutex_lock(&mutex) != 0)
-   {
-   CfLog(cferror,"pthread_mutex_lock failed","pthread_mutex_lock");
-   CfGetFile(args);
-   return;
-   }
-
-for (tid = 0; tid < MAXTHREADS; tid++)
-   {
-   if (THREADS[tid] != NULL)
-      {
-      count++;
-      }
-   }
- 
-if ((count >= CFD_MAXPROCESSES) || (count >= MAXTHREADS))
+if (ACTIVE_THREADS >= CFD_MAXPROCESSES)
    {
    CfLog(cfinform,"Too many threads","");
-   if (pthread_mutex_unlock(&mutex) != 0)
-      {
-      CfLog(cferror,"pthread_mutex_unlock failed","unlock");
-      }
-   return;
-   }
-
-
-for (tid = 0; tid < MAXTHREADS; tid++)
-   {
-   if (THREADS[tid] == NULL)
-      {
-      break;
-      }
-   }
-
-Note the threads share the stack, so args shouldn't be local vars which can deallocate 
- 
-if (pthread_create(&(THREADS[tid]),NULL,CfGetFile,args) != 0)
-   {
-   CfLog(cferror,"pthread_create failed","create");
    CfGetFile(args);
    return;
    }
 
-if (pthread_mutex_unlock(&mutex) != 0)
+if (MULTITHREAD)
    {
-   CfLog(cferror,"pthread_mutex_unlock failed","unlock");
+   Debug("Spawning new thread...\n");
+   
+   pthread_attr_init(&PTHREADDEFAULTS);
+   pthread_attr_setdetachstate(&PTHREADDEFAULTS,PTHREAD_CREATE_DETACHED);
+   
+   if (MULTITHREAD && (pthread_create(&tid,&PTHREADDEFAULTS,CfGetFile,args) != 0))
+      {
+      CfLog(cferror,"pthread_create failed","create");
+      CfGetFile(args);
+      }
+   
+
+   pthread_attr_destroy(&PTHREADDEFAULTS);
+   }
+else
+   {
+   CfGetFile(args);
+   return;
    }
 
-Careful! This will close the file descriptor before it cam be used...
-   
- */
- 
-CfGetFile(args);
+Debug("Thread done.\n"); 
  
 #else
-
+ 
+if ((!NO_FORK) && (fork() != 0))
+   {
+   return;
+   }
+ 
 CfGetFile(args);
 
-#endif 
+#endif
+
 }
 
 /**************************************************************/
@@ -1276,8 +1414,8 @@ if (strlen(filename) > maxlinksize)
 
 if (lstat(filename,&statbuf) == -1)
    {
-   sprintf(sendbuffer,"BAD: unable to stat file %s\n",filename);
-   CfLog(cferror,sendbuffer,"lstat");
+   sprintf(sendbuffer,"BAD: unable to stat file %s",filename);
+   CfLog(cfverbose,sendbuffer,"lstat");
 
    if (send(sd,sendbuffer,bufsize,0) == -1)
       {
@@ -1320,11 +1458,11 @@ if (S_ISLNK(statbuf.st_mode))
 if (stat(filename,&statbuf) == -1)
    {
    sprintf(sendbuffer,"BAD: unable to stat file %s\n",filename);
-   CfLog(cferror,OUTPUT,"stat");
+   CfLog(cfverbose,OUTPUT,"stat");
 
    if (send(sd,sendbuffer,bufsize,0) == -1)
       {
-      CfLog(cferror,"","send");
+      CfLog(cferror,"sendbuffer","send");
       }
          
    return -1;
@@ -1397,13 +1535,13 @@ bzero(sendbuffer,bufsize);
 
  /* send as plain text */
 
-Debug("OK: type=%d\n mode=%o\n lmode=%o\n uid=%d\n gid=%d\n size=%d\n atime=%d\n mtime=%d\n",
-	cfst.cf_type,cfst.cf_mode,cfst.cf_lmode,cfst.cf_uid,cfst.cf_gid,cfst.cf_size,
+Debug("OK: type=%d\n mode=%o\n lmode=%o\n uid=%d\n gid=%d\n size=%ld\n atime=%d\n mtime=%d\n",
+	cfst.cf_type,cfst.cf_mode,cfst.cf_lmode,cfst.cf_uid,cfst.cf_gid,(long)cfst.cf_size,
 	cfst.cf_atime,cfst.cf_mtime);
 
 
-sprintf(sendbuffer,"OK: %d %d %d %d %d %d %d %d %d %d %d %d",
-	cfst.cf_type,cfst.cf_mode,cfst.cf_lmode,cfst.cf_uid,cfst.cf_gid,cfst.cf_size,
+sprintf(sendbuffer,"OK: %d %d %d %d %d %ld %d %d %d %d %d %d",
+	cfst.cf_type,cfst.cf_mode,cfst.cf_lmode,cfst.cf_uid,cfst.cf_gid,(long)cfst.cf_size,
 	cfst.cf_atime,cfst.cf_mtime,cfst.cf_ctime,cfst.cf_makeholes,cfst.cf_ino,cfst.cf_nlink);
 
 if (send(sd,sendbuffer,bufsize,0) == -1)
@@ -1613,10 +1751,30 @@ void *CfGetFile(args)
 struct cfd_thread_arg *args;
 
 { int sd,fd, n_read;
-  char *sendbuffer, *filename;
+  char sendbuffer[bufsize], *filename;
 
-sd         = args->sd_reply;
-sendbuffer = args->replybuff;
+#if defined HAVE_LIBPTHREAD && defined NOTBROKEN  
+  pthread_mutex_t mutex;
+
+if (MULTITHREAD && (pthread_mutex_lock(&mutex) != 0))
+   {
+   CfLog(cferror,"pthread_mutex_lock failed","pthread_mutex_lock");
+   free(args->replyfile);  /* from strdup in each thread */
+   DeleteConn(args->connect);
+   free((char *)args);
+   return NULL;
+   }
+
+ACTIVE_THREADS++;
+
+if (MULTITHREAD && (pthread_mutex_unlock(&mutex) != 0))
+   {
+   CfLog(cferror,"pthread_mutex_unlock failed","unlock");
+   }
+  
+#endif
+
+sd         = (args->connect)->sd_reply;
 filename   = args->replyfile;
 
 Debug("CfGetFile(%s on sd=%d)\n",filename,sd);
@@ -1625,35 +1783,59 @@ if ((fd = open(filename,O_RDONLY)) == -1)
    {
    sprintf(sendbuffer,"BAD: can't access %s on server!\n",filename);
    CfLog(cferror,sendbuffer,"open");
-   return NULL;
    }
-
-while(true)
+else
    {
-   bzero(sendbuffer,bufsize);
-
-   if ((n_read = read(fd,sendbuffer,bufsize)) == -1)
+   while(true)
       {
-      close(fd);
-      sprintf(sendbuffer,"BAD: CfGetFile, read failed\n");
-      CfLog(cferror,sendbuffer,"read");
-      return NULL;
-      }
-
-   if (n_read == 0)
-      {
-      break;
-      }
-
-   if (send(sd,sendbuffer,n_read,0) == -1)
-      {
-      close(fd);
-      CfLog(cferror,"","send");
-      return NULL;
+      bzero(sendbuffer,bufsize);
+      
+      if ((n_read = read(fd,sendbuffer,bufsize)) == -1)
+	 {
+	 close(fd);
+	 CfLog(cferror,"read failed in GetFile","read");
+	 break;
+	 }
+      
+      if (n_read == 0)
+	 {
+	 break;
+	 }
+      
+      if (send(sd,sendbuffer,n_read,0) == -1)
+	 {
+	 close(fd);
+	 CfLog(cferror,"Send failed in GetFile","send");
+	 break;
+	 }
       }
+   
+   close(fd);
    }
 
-close(fd);
+ 
+DeleteConn(args->connect);
+ 
+free(args->replyfile);  /* from strdup in each thread */ 
+free((char *)args);
+ 
+#if defined HAVE_LIBPTHREAD && defined NOTBROKEN
+ 
+if (pthread_mutex_lock(&mutex) != 0)
+   {
+   CfLog(cferror,"pthread_mutex_lock failed","pthread_mutex_lock");
+   return;
+   }
+ 
+ACTIVE_THREADS--;
+   
+if (pthread_mutex_unlock(&mutex) != 0)
+   {
+   CfLog(cferror,"pthread_mutex_unlock failed","unlock");
+   }
+
+#endif 
+
 return NULL;
 }
 
@@ -1665,7 +1847,7 @@ struct cfd_connection *conn;
 
 { char buffer[bufsize];
 
-sprintf(buffer,"Hello %s, nothing to be done on %s\n",conn->hostname,VFQNAME);
+sprintf(buffer,"Hello %s, nothing to do here...\n",conn->hostname,VFQNAME);
 
 if (send(conn->sd_reply,buffer,bufsize,0) == -1)
    {
@@ -1681,13 +1863,15 @@ char *filename;
 char digest[17];
 
 {     
-#if defined HAVE_LIBDB && (defined HAVE_DB_H || defined HAVE_DB_DB_H)
+#if defined HAVE_LIBDB && (defined HAVE_DB_H || defined HAVE_DB_DB_H) && defined HAVE_DB_185_H
   struct stat stat1, stat2;
   int i, uptodate = true;
-  int result = 0;
   DBT key,value;
-  DB *db;
-
+  DB *dbp;
+  DBC *dbcp;
+  DB_INFO dbinfo;
+  db_recno_t recno;
+	
 Debug("InChecksumDB(%s from %s)\n",filename,CHECKSUMDB);
   
 if (stat(filename,&stat1) == -1)
@@ -1708,31 +1892,40 @@ if (CHECKSUMDB[0] == '\0')
    Verbose("cfd: No cache database defined\n");
    return false;
    }
+ 
+ /* Initialize the database. */
+ memset(&dbinfo, 0, sizeof(dbinfo));
+ dbinfo.db_cachesize = 32 * 1024;	/* Cachesize: 32K. */
+ dbinfo.db_pagesize = 1024;		/* Page size: 1K. */
+ 
+ /* Create the database. */
+ if ((errno = db_open(CHECKSUMDB,DB_BTREE, DB_CREATE, 0644, NULL, &dbinfo, &dbp)) != 0)
+    {
+    sprintf(OUTPUT,"cfd: couldn't open checksum database %s\n",CHECKSUMDB);
+    CfLog(cferror,OUTPUT,"db_open");
+    return false;
+    }
 
-if ((db = dbopen(CHECKSUMDB, O_RDWR | O_CREAT, 0644, DB_BTREE,NULL)) == NULL)
-   {
-   printf("cfd: couldn't open checksum database %s\n",CHECKSUMDB);
-   CfLog(cferror,"","db_open");
-   return false;
-   }
-
+bzero(&value,sizeof(value)); 
+bzero(&key,sizeof(key));       
+      
 key.data = filename;
 key.size = strlen(filename)+1;
-      
-if ((result = db->get(db,&key,&value,0)) == 0)
+ 
+if ((errno = dbp->get(dbp,NULL,&key,&value,0)) == 0)
    {
    if (uptodate)
       {
       Debug("cfd: Found checksum for %s in database\n",filename);
       bcopy(value.data,digest,17);
-      db->close(db);
+      dbp->close(dbp,0);
       return true;
       }
    else
       {
       cfMDFile(filename,digest);
       digest[16] = '\0';
-
+              
       key.data = filename;
       key.size = strlen(filename)+1;
       value.data = (void *) digest;
@@ -1740,7 +1933,7 @@ if ((result = db->get(db,&key,&value,0)) == 0)
 
       Debug("cfd: updating checksum for %s\n",filename);
       
-      if (db->del(db,&key,0) != 0)
+      if ((errno = dbp->del(dbp,NULL,&key,0)) != 0)
 	 {
 	 CfLog(cferror,"","db_store");
 	 }
@@ -1748,19 +1941,17 @@ if ((result = db->get(db,&key,&value,0)) == 0)
       key.data = filename;
       key.size = strlen(filename)+1;
 
-      if ((result = db->put(db,&key,&value,0)) != 0)
+      if ((errno = dbp->put(dbp,NULL,&key,&value,0)) != 0)
 	 {
-	 Debug("retrun val = %d\n",result);
-	 CfLog(cferror,"","db_store");
+	 CfLog(cferror,"put failed","db->put");
 	 }
 
-      db->close(db);
+      dbp->close(dbp,0);
       return true;
       }
    }
 else
    {
-   Debug("result= %d\n",result);
    CfLog(cferror,"","get");
    cfMDFile(filename,digest);
    digest[16] = '\0';
@@ -1772,13 +1963,12 @@ else
 
    Debug("cfd: storing checksum for %s in database\n",filename);
 
-   if ((result = db->put(db,&key,&value,0)) != 0)
+   if ((errno = dbp->put(dbp,NULL,&key,&value,0)) != 0)
       {
-      printf("return value = %d\n",result);
-      CfLog(cferror,"","db_store");
+      CfLog(cferror,"put failed","db->put");
       }
    
-   db->close(db);
+   dbp->close(dbp,0);
    return true;   
    }
 
@@ -1789,6 +1979,36 @@ return false;
 }
 
 
+/***************************************************************/
+/* Toolkit/Class: conn                                         */
+/***************************************************************/
+
+struct cfd_connection *NewConn(sd)  /* construct */
+
+int sd;
+
+{ struct cfd_connection *conn;
+
+conn = (struct cfd_connection *) malloc(sizeof(struct cfd_connection));
+
+conn->sd_reply = sd;
+conn->id_verified = false;
+conn->hostname[0] = '\0'; 
+ 
+return conn;
+}
+
+/***************************************************************/
+
+DeleteConn(conn) /* destruct */
+
+struct cfd_connection *conn;
+
+{
+close(conn->sd_reply);
+free ((char *)conn);
+}
+
 /* EOF */
 
 
