@@ -64,14 +64,13 @@ struct cfd_connection
    {
    int id_verified;
    int synchronized;
-   int sd_recv;
    int sd_reply;
-   char *hostname;
+   char hostname[maxvarsize];
    };
 
 struct cfd_thread_arg
    {
-   int sd_reply;
+   struct cfd_connection *connect;
    char *replybuff;
    char *replyfile;
    };
@@ -82,7 +81,10 @@ struct cfd_thread_arg
 
 int CLOCK_DRIFT = 100;
 int CFD_MAXPROCESSES = 0;
+int ACTIVE_THREADS = 0;
 int NO_FORK = false;
+int MULTITHREAD = false;
+int CFD_INTERVAL = 0;
 
 struct option CFDOPTIONS[] =
    {
@@ -91,6 +93,7 @@ struct option CFDOPTIONS[] =
    { "verbose",no_argument,0,'v' },
    { "no-fork",no_argument,0,'F' },
    { "parse-only",no_argument,0,'p'},
+   { "multithread",no_argument,0,'m'},
    { NULL,0,0,0 }
    };
 
