@@ -28,14 +28,27 @@
 /*  Cfengine : remote client example                               */
 /*                                                                 */
 /*  Mark Burgess 1997                                              */
+/*  modified : Bas van der Vlies <basv@sara.nl> 1998               */
 /*                                                                 */
 /*******************************************************************/
 
 #define INET 1
 
+/*
+ * Bas 
+*/
+#define FileVerbose if (VERBOSE || DEBUG || D2) fprintf
+
 #include "cf.defs.h"
 #include "cf.extern.h"
 
+/* 
+ * Bas
+*/
+int  MAXCHILD = 1;
+int  FileFlag = 0;
+char OUTPUTDIR[bufsize];
+
 struct Item *VCFRUNCLASSES = NULL;
 struct Item *VCFRUNHOSTLIST = NULL;
 
@@ -53,14 +66,54 @@ int argc;
 char **argv;
 
 { struct Item *ip;
+  int 	 i=0;
+  int	 status;
+  int	 pid;
 
 CheckOptsAndInit(argc,argv);
 
-for (ip = VCFRUNHOSTLIST; ip != NULL; ip=ip->next)
-   {
-   printf("cfrun: (%s)\n",ip->name);
-   ConnectToServer(ip->name,ip->classes);
-   }
+/*
+ * Bas
+*/
+ip = VCFRUNHOSTLIST;
+while (ip != NULL )
+{
+
+ if (i < MAXCHILD)
+    {
+    if ( fork() == 0  ) /* child */
+       {
+       printf("cfrun: (%s) Thread %d\n",ip->name, i);
+       Debug("pid = %d i = %d\n", getpid(), i);
+       ConnectToServer(ip->name,ip->classes, FileFlag);
+       exit(0);
+       }
+    else
+       {
+       /* parent */
+       i++;
+       }
+    }
+ else 
+    {
+    pid = wait(&status);
+    Debug("wait result pid = %d number %d\n", pid, i);
+    i--;
+    }
+ if ( i < MAXCHILD )
+    {
+    ip = ip->next;
+    }
+}
+ 
+ while (i > 0 )
+    {
+    pid = wait(&status);
+    Debug("Child pid = %d ended\n", pid);
+    i--;
+    }
+ 
+ exit(0);
 }
 
 /********************************************************************/
@@ -72,7 +125,8 @@ CheckOptsAndInit(argc,argv)
 int argc;
 char **argv;
 
-{ char buffer[maxvarsize], options[bufsize], line[bufsize], filename[bufsize], *sp;
+{ char buffer[maxvarsize], options[bufsize], line[bufsize], filename[bufsize], *
+sp;
   char id[maxvarsize]; 
   struct Item *ip;
   FILE *fp;
@@ -113,6 +167,30 @@ while (!feof(fp))
       continue;
       }
 
+   if (strncmp(line,"maxchild", strlen("maxchild")) == 0)
+      {
+      sscanf(line,"maxchild = %[^# \n]", buffer);
+
+      if ( (MAXCHILD = atoi(buffer)) == 0 )
+	MAXCHILD = 1;
+
+      Verbose("maxchild = %d\n", MAXCHILD);
+      continue;
+      }
+
+   if (strncmp(line,"outputdir", strlen("outputdir")) == 0)
+      {
+      sscanf(line,"outputdir = %[^# \n]", OUTPUTDIR);
+      if ( opendir(OUTPUTDIR) == NULL)
+	 {
+         printf("Directory %s does not exists\n", OUTPUTDIR);
+	 exit(1);
+         }
+
+      FileFlag=1;
+      continue;
+      }
+
    if (strncmp(line,"access",6) == 0)
       {
       for (sp = line; (*sp != '=') && (*sp != '\0'); sp++)
@@ -222,25 +300,41 @@ else
 
 /********************************************************************/
 
-ConnectToServer(host,options)
+ConnectToServer(host,options, StoreInFile)
 
 char *host, *options;
+int  StoreInFile;
 
 { struct hostent *hp;
   struct sockaddr_in cin, local;
   char sendbuffer[bufsize];
   char recvbuffer[bufsize];
+  char filebuffer[bufsize];
+
   struct servent *server;
   int sd,i,err,TimeOut(),offset,n_read, len;
   char *sp;
+  FILE  *fp;
 
-Verbose("Connecting to server %s with options %s %s\n",host,options,CFRUNOPTIONS);
+
+  if (StoreInFile)
+  {
+    sprintf(filebuffer, "%s/%s", OUTPUTDIR, host);
+    fp = fopen(filebuffer, "w");
+  }
+  else
+    fp = stdout;
+
+  FileVerbose(fp, "Connecting to server %s with options %s %s\n",
+			host,options,CFRUNOPTIONS);
 
 if ((hp = gethostbyname(host)) == NULL)
    {
    printf("Unknown host: %s\n", host);
-   printf("Make sure that fully qualified names can be looked up at your site!\n");
-   printf("i.e. prep.ai.mit.edu, not just prep. If you use NIS or /etc/hosts\n");
+   printf("Make sure that fully qualified names can be looked up at your site!\n
+");
+   printf("i.e. prep.ai.mit.edu, not just prep. If you use NIS or /etc/hosts\n")
+;
    printf("make sure that the full form is registered too as an alias!\n");
    exit(1);
    }
@@ -253,7 +347,8 @@ if ((server = getservbyname(CFENGINE_SERVICE,"tcp")) == NULL)
 
 bzero(&cin,sizeof(cin));
 
-cin.sin_port = (unsigned int) server->s_port; /* htons((unsigned short)(server->s_port)); */
+cin.sin_port = (unsigned int) server->s_port; /* htons((unsigned short)(server->
+s_port)); */
 cin.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;
 cin.sin_family = AF_INET; 
 
@@ -287,7 +382,8 @@ if (getsockname(sd,(struct sockaddr *)&local,&len) == -1)
    return false;
    }
 
-Verbose("Identifying as %s as %s\n",inet_ntoa(local.sin_addr),VFQNAME);
+FileVerbose(fp, "Identifying as %s as %s\n", 
+		     inet_ntoa(local.sin_addr), VFQNAME);
 
 if (! IdentifyForVerification(sd,inet_ntoa(local.sin_addr),VFQNAME))
    {
@@ -308,7 +404,7 @@ if (send(sd,sendbuffer,bufsize,0) == -1)
 
 SendClassData(sd,sendbuffer);
 
-Verbose("\n%s replies..\n\n",host);
+FileVerbose(fp, "\n%s replies..\n\n",host);
 
 while (true)
    {
@@ -328,7 +424,7 @@ while (true)
    if ((sp = strstr(recvbuffer,CFD_TERMINATOR)) != NULL)
       {
       *sp = '\0';
-      printf("%s\n",recvbuffer);
+      fprintf(fp,"%s",recvbuffer);
       break;
       }
 
@@ -349,15 +445,16 @@ while (true)
 
    if (strstr(recvbuffer,"cfd:")) /* Don't print messages fom cfd unless -v */
       {
-      Verbose("%s",recvbuffer);
+      FileVerbose(fp, "%s",recvbuffer);
+      sleep(1);
       continue;
       }
 
-   printf("%s",recvbuffer);
+    fprintf(fp,"%s",recvbuffer);
+    FileVerbose(fp,"\n==========================================================
+==============\n");
    }
 
-Verbose("\n========================================================================\n");
-
 close(sd);
 return true;
 }
