@@ -200,7 +200,7 @@ char *VMOUNTCOMM[clssattr] =
    "/sbin/mount -va",     /* irix4 */
    "/sbin/mount -va",     /* irix */
    "/sbin/mount -va",     /* irix64 */
-   "/sbin/mount -va" ,    /* freebsd */
+   "/sbin/mount -u -va",  /* freebsd */
    "/usr/sbin/mount -a",  /* solarisx86 */
    "/etc/mount -a",       /* bsd 4.3 */
    "/etc/mount -a",       /* newsos4 */
@@ -229,7 +229,7 @@ char *VUNMOUNTCOMM[clssattr] =
    "/sbin/umount",     /* hpux10 */
    "/etc/umount",      /* hpux */
    "/etc/umount",      /* aix */
-   "/sbin/umount",     /* linux */
+   "/bin/umount",      /* linux */
    "/etc/umount",      /* solaris */
    "/usr/sbin/umount", /* osf1 */
    "/etc/umount",      /* sun3 */
