@@ -191,8 +191,8 @@ if (strncmp(recvbuffer,"OK:",3) == 0)
    Debug("RECV:%s\n",recvbuffer);
    Debug("Mode = %d,%d\n",d2,d3);
    
-   Debug("OK: type=%d\n mode=%o\n lmode=%o\n uid=%d\n gid=%d\n size=%d\n atime=%d\n mtime=%d ino=%d nlnk=%d\n",
-	cfst.cf_type,cfst.cf_mode,cfst.cf_lmode,cfst.cf_uid,cfst.cf_gid,cfst.cf_size,
+   Debug("OK: type=%d\n mode=%o\n lmode=%o\n uid=%d\n gid=%d\n size=%ld\n atime=%d\n mtime=%d ino=%d nlnk=%d\n",
+	cfst.cf_type,cfst.cf_mode,cfst.cf_lmode,cfst.cf_uid,cfst.cf_gid,(long)cfst.cf_size,
 	cfst.cf_atime,cfst.cf_mtime,cfst.cf_ino,cfst.cf_nlink);
 
    bzero(recvbuffer,bufsize);
