@@ -64,6 +64,9 @@ enum cfsizes type;
 
     bzero (&buf, sizeof (buf));
 
+    Verbose("Checking free space on %s\n",file);
+
+
 #if defined ULTRIX
     if (getmnt (NULL, &buf, sizeof (struct fs_data), STAT_ONE, file) == -1)
        {
@@ -138,7 +141,7 @@ enum cfsizes type;
     
 capacity = (double) (avail) / (double) (avail + used) * 100;
 
-Debug2("GetDiskUsage(%s)\n",file);
+Debug2("GetDiskUsage(%s) = %d/%d\n",file,avail,capacity);
     
 /* Free kilobytes */
 
