@@ -162,7 +162,8 @@ currenteditscript = NULL;
 searchstr = NULL;
 bzero(ACTIONBUFF,bufsize);
 AUTOCREATED = false;
- 
+IMAGEBACKUP = true;
+
 for (ep = ptr->actions; ep != NULL; ep=ep->next)
    {
    if (!IsExcluded(ep->classes))
@@ -181,7 +182,7 @@ if (!GetLock(ASUniqueName("editfile"),CanonifyName(ptr->fname),VIFELAPSED,VEXPIR
    return;
    }
 
-CheckCreate(filename,ptr->actions);
+CheckEditSwitches(filename,ptr->actions);
 
 if (! LoadItemList(&filestart,filename))
    {
@@ -821,6 +822,7 @@ if ((! DONTDO) && (NUMBEROFEDITS > 0))
    AddEditfileClasses(ptr);
    }
 
+ResetOutputRoute('d','d');
 ReleaseCurrentLock();
 
 DeleteItemList(filestart);
@@ -1128,13 +1130,14 @@ for (ip = VMOUNTABLES; ip != NULL; ip=ip->next)
 
 /**************************************************************/
 
-CheckCreate(filename,actions)
+CheckEditSwitches(filename,actions)
 
 char *filename;
 struct Edlist *actions;
 
 { struct stat statbuf;
   struct Edlist *ep;
+  char inform = 'd', log = 'd';
   int fd;
 
 for (ep = actions; ep != NULL; ep=ep->next)
@@ -1162,8 +1165,43 @@ for (ep = actions; ep != NULL; ep=ep->next)
 			  CfLog(cfinform,OUTPUT,"");
 			  return;
 	                  }
+                       break;
+      
+      case EditBackup: if (strcmp("false",ToLowerStr(ep->data)) == 0 || strcmp("off",ToLowerStr(ep->data)) == 0)
+	                  {
+			  IMAGEBACKUP = false;
+	                  }
+                       break;
+		       
+      case EditLog:    if (strcmp(ToLowerStr(ep->data),"true") == 0 || strcmp(ToLowerStr(ep->data),"on") == 0)
+	                  {
+			  log = 't';
+			  break;
+			  }
+
+                       if (strcmp(ToLowerStr(ep->data),"false") == 0 || strcmp(ToLowerStr(ep->data),"off") == 0)
+                          {
+			  log = 'f';
+			  break;
+			  }
+
+      case EditInform: if (strcmp(ToLowerStr(ep->data),"true") == 0 || strcmp(ToLowerStr(ep->data),"on") == 0)
+	                  {
+			  inform = 't';
+			  break;
+			  }
+
+                       if (strcmp(ToLowerStr(ep->data),"false") == 0 || strcmp(ToLowerStr(ep->data),"off") == 0)
+                          {
+			  inform = 'f';
+			  break;
+			  }
+
+	  
       }
    }
+
+ResetOutputRoute(log,inform);
 }
 
 
