@@ -79,13 +79,13 @@ char *classlist;
 
 { char *sp, currentitem[maxvarsize];
 
-Debug("AddInstallable(%s)\n",classlist);
- 
 if (classlist == NULL)
    {
    return;
    }
 
+Debug("AddInstallable(%s)\n",classlist);
+  
  for (sp = classlist; *sp != '\0'; sp++)
    {
    currentitem[0] = '\0';
@@ -129,7 +129,7 @@ for (sp = classlist; *sp != '\0'; sp++)
 
    sp += strlen(currentitem);
 
-   AddClassToHeap(currentitem);
+   AddClassToHeap(CanonifyName(currentitem));
    }
  
 }
@@ -351,6 +351,11 @@ IsDefinedClass(class)
 char *class;
 
 {
+if (class == NULL)
+   {
+   return true;
+   }
+ 
 return EvaluateORString(class,VADDCLASSES);
 }
 
@@ -365,7 +370,7 @@ char *class;
   /* the course of the execution - but might not be true now    */
 
 {
-return EvaluateORString(class,VALLADDCLASSES);
+return (EvaluateORString(class,VALLADDCLASSES)||EvaluateORString(class,VADDCLASSES));
 }
 
 
@@ -414,7 +419,7 @@ struct Item **heap;
 { char *sp = class;
   char cbuff[maxvarsize];
 
-Debug1("AddCompoundClass(%s)",class);
+Debug1("NegateCompoundClass(%s)",class);
 
 while(*sp != '\0')
    {
