@@ -38,24 +38,43 @@ char *name;
 
 { char *sp;
   struct Item *ip;
-
+  int slashes;
+  
 if (VMOUNTLIST == NULL)
    {
    return (false);
-   }
-
-for (sp = name+strlen(name); *(sp-1) != '/' && sp >= name; sp--)
-   {
-   }
+   } 
 
 for (ip = VHOMEPATLIST; ip != NULL; ip=ip->next)
    {
+   slashes = 0;
+
+   for (sp = ip->name; *sp != '\0'; sp++)
+      {
+      if (*sp == '/')
+	 {
+	 slashes++;
+	 }
+      }
+   
+   for (sp = name+strlen(name); (*(sp-1) != '/') && (sp >= name) && (slashes >= 0); sp--)
+      {
+      if (*sp == '/')
+	 {
+	 slashes--;
+	 }
+      }
+
+   /* Full comparison */
+   
    if (WildMatch(ip->name,sp))
       {
+      Debug("IsHomeDir(true)\n");
       return(true);
       }
    }
 
+Debug("IsHomeDir(false)\n");
 return(false);
 }
 
@@ -117,12 +136,14 @@ if (recurse == -1)
    return;
    }
 
-Debug("RecursiveCheck(%s)\n",name);
+Debug("RecursiveCheck(%s,+%o,-%o)\n",name,plus,minus);
 
 if ((dirh = opendir(name)) == NULL)
    {
-   sprintf(OUTPUT,"Can't open directory %s\n",name);
-   CfLog(cferror,OUTPUT,"opendir");
+   if (lstat(name,&statbuf) != -1)
+      {
+      CheckExistingFile(name,plus,minus,action,uidlist,gidlist,&statbuf,ptr,ptr->acl_aliases);
+      }
    return;
    }
 
@@ -176,13 +197,15 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
 
    if (S_ISDIR(statbuf.st_mode))
       {
-      if (IsMountedFileSystem(&statbuf,pcwd,rlevel) || (recurse-1 == 0))
+      Debug("File is a directory\n");
+      
+      if (IsMountedFileSystem(&statbuf,pcwd,rlevel))
          {
          continue;
          }
       else
          {
-         if (strcmp(dirp->d_name,".") != 0)  /* avoid infinite regression */
+         if (ptr->recurse > 1)  /* avoid infinite regression */
             {
             CheckExistingFile(pcwd,plus,minus,action,uidlist,gidlist,&statbuf,ptr,ptr->acl_aliases);
             RecursiveCheck(pcwd,plus,minus,action,uidlist,gidlist,recurse-1,rlevel+1,ptr);
@@ -220,6 +243,7 @@ struct Item *acl_aliases;
   int amroot = true;
 
 Verbose("%s: Checking fs-object %s\n",VPREFIX,file);
+Debug("CheckExistingFile(+%o,-%o)\n",plus,minus); 
 
 if (ptr != NULL)
    {
@@ -244,6 +268,7 @@ newperm &= ~minus;
 
 if (S_ISREG(dstat->st_mode) && (action == fixdirs || action == warndirs)) 
    {
+   Debug("Regular file, returning...\n");
    return;
    }
 
@@ -254,17 +279,19 @@ if (S_ISDIR(dstat->st_mode))
       return;
       }
 
-   if (dstat->st_mode & S_IRUSR)
+   Debug("Directory...fixing x bits\n");
+
+   if (newperm & S_IRUSR)
       {
       newperm  |= S_IXUSR;
       }
 
-   if (dstat->st_mode & S_IRGRP)
+   if (newperm & S_IRGRP)
       {
       newperm |= S_IXGRP;
       }
 
-   if (dstat->st_mode & S_IROTH)
+   if (newperm & S_IROTH)
       {
       newperm |= S_IXOTH;
       }
@@ -378,7 +405,9 @@ if (S_ISLNK(dstat->st_mode))             /* No point in checking permission on a
 
 if (stat(file,dstat) == -1)
    {
-   perror("stat");
+   sprintf(OUTPUT,"Can't stat file %s while looking at permissions (was not copied?)\n",file);
+   CfLog(cfverbose,OUTPUT,"stat");
+   return;
    }
 
 if (CheckACLs(file,action,acl_aliases))
@@ -391,9 +420,12 @@ if (CheckACLs(file,action,acl_aliases))
  
 if (((newperm & 07777) == (dstat->st_mode & 07777)) && (action != touch))    /* file okay */
    {
+   Debug("File okay, newperm = %o, stat = %o\n",(newperm & 07777),(dstat->st_mode & 07777));
    return;
    }
 
+Debug("Trying to fix...\n"); 
+
 switch (action)
    {
    case linkchildren:
@@ -510,6 +542,8 @@ switch (action)
 
    default:     FatalError("cfengine: internal error CheckExistingFile(): illegal file action\n");
    }
+
+Debug("CheckExistingFile(Done)\n"); 
 }
 
 /*********************************************************************/
@@ -529,23 +563,14 @@ struct Item *acl_aliases;
 { mode_t newplus,newminus;
 
  /* plus/minus must be relative to source file, not to
-    perms of newly created file!
+    perms of newly created file! */
 
+Debug("CheckCopiedFile(%s)\n",file); 
 
-! if ((plus == 0) && (minus == 0))
-     {
-!    newplus = sstat->st_mode & 07777;
-!    newminus = ~(sstat->st_mode & 07777) & 07777;
-     CheckExistingFile(file,newplus,newminus,fixall,uidlist,gidlist,dstat,NULL,acl_aliases);
-     }
-
- */
-
- if (true)
+if ((plus == 0) && (minus == 0))
     {
     newplus = sstat->st_mode & 07777 | plus;
     newminus = ~(sstat->st_mode & 07777 & ~minus) & 07777;
-    Debug2("CheckCopiedFile: %s: +%o, -%o\n",file,newplus,newminus);
     CheckExistingFile(file,newplus,newminus,fixall,uidlist,gidlist,dstat,NULL,acl_aliases);
     }
  else
@@ -642,11 +667,13 @@ else
                         if (uid != -1)
                            {
                            sprintf(OUTPUT,"  (Changing owner to uid %d)\n",uid);
+                           CfLog(cfinform,OUTPUT,"");
                            }
 
                         if (gid != -1)
                            {
                            sprintf(OUTPUT,"  (Changing group to gid %d)\n",gid);
+                           CfLog(cfinform,OUTPUT,"");
                            }
                         }
                      }
@@ -654,6 +681,7 @@ else
                   if (! DONTDO && S_ISLNK(statbuf->st_mode))
                      {
 #ifdef HAVE_LCHOWN
+		     Debug("Using LCHOWN function\n");
                      if (lchown(file,uid,gid) == -1)
                         {
                         sprintf(OUTPUT,"Cannot set ownership on link %s!\n",file);
@@ -710,9 +738,10 @@ return false;
 
 /*********************************************************************/
 
-CheckHomeSubDir(testpath,tidypath)
+CheckHomeSubDir(testpath,tidypath,recurse)
 
 char *testpath, *tidypath;
+int recurse;
 
 { char *sp, *subdirstart, *sp1, *sp2;
   char buffer[bufsize];
