@@ -337,10 +337,19 @@ char *nodename, *path;
   struct stat statbuf; 
   char *sp;
 
+Debug("SensibleFile(%s,%s)\n",path,nodename);
+  
+if (strcmp(nodename,"...") == 0 && strcmp(path,"/") == 0)
+   {
+   Verbose("DFS cell node detected in /...\n");
+   return;
+   }
+  
 for (i = 0; VSKIPFILES[i] != NULL; i++)
    {
    if (strcmp(nodename,VSKIPFILES[i]) == 0)
       {
+      Debug("Filename is classified as ignorable\n");
       return false;
       }
    }
@@ -349,6 +358,7 @@ for (sp = nodename; *sp != '\0'; sp++) /* Check for files like ".. ." */
    {
    if ((*sp != '.') && ! isspace(*sp))
       {
+      Debug("Filename is not suspicious\n");
       suspicous = false;
       return true;
       }
@@ -356,6 +366,7 @@ for (sp = nodename; *sp != '\0'; sp++) /* Check for files like ".. ." */
 
 sprintf(OUTPUT,"Suspicous looking file object \"%s\" masquerading as hidden file in %s\n",nodename,path);
 CfLog(cfsilent,OUTPUT,"");
+Debug("Filename looks suspicious\n"); 
 
 strcpy(VBUFF,path);
 AddSlash(VBUFF);
