@@ -92,6 +92,8 @@ char OUTPUT[bufsize*2];
   PUBLIC short VERBOSE = false;
   PUBLIC short INFORM = false;
   PUBLIC short LOGGING = false;
+  PUBLIC short INFORM_save;
+  PUBLIC short LOGGING_save;
 
   PUBLIC char VINPUTFILE[bufsize];
   PUBLIC char VCURRENTFILE[bufsize];
@@ -102,7 +104,7 @@ char OUTPUT[bufsize*2];
   PUBLIC char VREPOSITORY[bufsize];
   PUBLIC char ALLCLASSBUFFER[bufsize];
 
-  PRIVATE   int RPCTIMEOUT = 60;          /* seconds */
+  PRIVATE   int RPCTIMEOUT = 30;          /* seconds */
   PROTECTED int SENSIBLEFILECOUNT = 2;
   PROTECTED int SENSIBLEFSSIZE = 1000;
 
@@ -130,6 +132,7 @@ char OUTPUT[bufsize*2];
  /* Data structures - root pointers                                 */
  /*******************************************************************/
 
+  PROTECTED  struct Item *VTIMEZONE = NULL;
   PROTECTED  struct Item *VMOUNTLIST = NULL;
   PROTECTED  struct Item *VEXCLUDECOPY = NULL;
   PROTECTED  struct Item *VEXCLUDELINK = NULL;
@@ -150,6 +153,8 @@ char OUTPUT[bufsize*2];
   PROTECTED  struct Disk *VREQUIREDTOP = NULL;
   PROTECTED  struct ShellComm *VSCRIPT = NULL;              /* List of scripts to execute */
   PROTECTED  struct ShellComm *VSCRIPTTOP = NULL;
+  PROTECTED  struct Interface *VIFLIST = NULL;
+  PROTECTED  struct Interface *VIFLISTTOP = NULL;
   PROTECTED  struct Mounted *MOUNTED = NULL;             /* Files systems already mounted */
   PROTECTED  struct MiscMount *VMISCMOUNT = NULL;
   PROTECTED  struct MiscMount *VMISCMOUNTTOP = NULL;
@@ -217,7 +222,6 @@ char OUTPUT[bufsize*2];
  PROTECTED char      VNETMASK[maxvarsize];
  PROTECTED char    VBROADCAST[maxvarsize];
  PROTECTED char   VMAILSERVER[bufsize];
- PROTECTED char     VTIMEZONE[maxvarsize];
  PROTECTED char VDEFAULTROUTE[maxvarsize];
  PROTECTED char      VNFSTYPE[maxvarsize];
  PROTECTED char       VFQNAME[maxvarsize];
@@ -272,6 +276,7 @@ char OUTPUT[bufsize*2];
       { "no-lock",no_argument,0,'K'},
       { "auto",no_argument,0,'A'},
       { "inform",no_argument,0,'I'},
+      { "no-modules",no_argument,0,'M'},
       { NULL,0,0,0 }
       };
 
@@ -310,6 +315,7 @@ char OUTPUT[bufsize*2];
       "Admit network access:",
       "Deny network access:",
       "Access control lists:",
+      "Additional network interfaces",
       NULL
       };
 
@@ -343,6 +349,7 @@ char OUTPUT[bufsize*2];
       "admit",
       "deny",
       "acl",
+      "interfaces",
       NULL
       };
 
@@ -411,6 +418,7 @@ char OUTPUT[bufsize*2];
   PRIVATE short PARSEONLY = false;
   PRIVATE short GOTMOUNTINFO = true;
   PRIVATE short NOMOUNTS = false;
+  PRIVATE short NOMODULES = false;
   PRIVATE short NOFILECHECK = false;
   PRIVATE short NOTIDY = false;
   PRIVATE short NOSCRIPTS = false;
@@ -451,6 +459,9 @@ char OUTPUT[bufsize*2];
   PRIVATE char DISCOMP = '=';
   PRIVATE char USESHELL = 'y';  /* yes or no */
   PRIVATE char PURGE = 'n';
+  PRIVATE char LOGP = 'd';  /* y,n,d=default*/
+  PRIVATE char INFORMP = 'd';
+  PRIVATE char MOUNTMODE = 'w';   /* o or w for rw/ro*/
 
   PRIVATE char VUIDNAME[maxvarsize];
   PRIVATE char VGIDNAME[maxvarsize];
@@ -468,6 +479,7 @@ char OUTPUT[bufsize*2];
   PRIVATE char DESTINATION[bufsize];
   PRIVATE char IMAGEACTION[bufsize];  /* end recyclable (see cf.defs.h) */
   PRIVATE char VIFNAME[16];
+  PRIVATE char VIFNAMEOVERRIDE[16];
 
   PUBLIC int ERRORCOUNT = 0;
   PUBLIC int LINENUMBER = 1;
@@ -531,6 +543,10 @@ char OUTPUT[bufsize*2];
      "acl",
      "purge",
      "useshell",
+     "syslog",
+     "inform",
+     "netmask",
+     "broadcast",
      NULL
      };
 
@@ -541,7 +557,7 @@ char OUTPUT[bufsize*2];
 /*                                                                 */
 /*******************************************************************/
 
-  PUBLIC int EDITFILESIZE = 1000;
+  PUBLIC int EDITFILESIZE = 10000;
 
   PRIVATE int NUMBEROFEDITS = 0;
   PRIVATE char VEDITABORT[maxlinksize];
@@ -639,6 +655,9 @@ char OUTPUT[bufsize*2];
      "DeleteLinesAfterThisMatching",
      "DefineClasses",
      "CatchAbort",
+     "Backup",
+     "Syslog",
+     "Inform",
      NULL
      };
 
