@@ -75,6 +75,8 @@
 
 struct ifreq IFR;
 
+char VNUMBROADCAST[256];
+
 #define cfproto 0
 
 #ifndef IPPROTO_IP     /* Old boxes, hpux 7 etc */
@@ -87,34 +89,34 @@ struct ifreq IFR;
 
 /*******************************************************************/
 
-IfConf (class)
+IfConf (vifdev,vnetmask,vbroadcast)
 
-enum classes class;
+char *vifdev,*vnetmask, *vbroadcast;
 
 { int sk, flags, metric, isnotsane = false;
 
-if (getuid() != 0)                            
+Verbose("Assumed interface name: %s %s %s\n",vifdev,vnetmask,vbroadcast);
+
+ if (getuid() != 0)                            
    {
    printf("%s: Only root can configure the net interface.\n",VPREFIX);
    return;
    }
 
-if (VNETMASK[0] == '\0')
+if (vnetmask[0] == '\0')
    {
    FatalError("Program does not define a subnetmask");
    }
 
-if (VBROADCAST[0] == '\0')
+if (vbroadcast[0] == '\0')
    {
    FatalError("Program does not define a broadcast mode for this host");
    }
 
 
-strcpy(IFR.ifr_name,VIFDEV[class]);
+strcpy(IFR.ifr_name,vifdev);
 IFR.ifr_addr.sa_family = AF_INET;
 
-Verbose("Interface name: %s\n",VIFDEV[class]);
-
 if ((sk = socket(AF_INET,SOCK_DGRAM,IPPROTO_IP)) == -1)
    {
    CfLog(cferror,"","socket");
@@ -128,7 +130,7 @@ if (ioctl(sk,SIOCGIFFLAGS, (caddr_t) &IFR) == -1)   /* Get the device status fla
    }
 
 flags = IFR.ifr_flags;
-strcpy(IFR.ifr_name,VIFDEV[class]);                   /* copy this each time */
+strcpy(IFR.ifr_name,vifdev);                   /* copy this each time */
  
 if (ioctl(sk,SIOCGIFMETRIC, (caddr_t) &IFR) == -1)   /* Get the routing priority */
    {
@@ -138,12 +140,12 @@ if (ioctl(sk,SIOCGIFMETRIC, (caddr_t) &IFR) == -1)   /* Get the routing priority
 
 metric = IFR.ifr_metric;
 
-isnotsane = GetIfStatus(sk,class);
+isnotsane = GetIfStatus(sk,vifdev,vnetmask,vbroadcast);
 
 if (! DONTDO && isnotsane)
    {
-   SetIfStatus(sk,class);
-   GetIfStatus(sk,class);
+   SetIfStatus(sk,vifdev,vnetmask,vbroadcast);
+   GetIfStatus(sk,vifdev,vnetmask,vbroadcast);
    }
 
 close(sk);
@@ -152,10 +154,10 @@ close(sk);
 
 /*******************************************************************/
 
-GetIfStatus(sk,class)
+GetIfStatus(sk,vifdev,vnetmask,vbroadcast)
 
 int sk;
-enum classes class;
+char *vifdev,*vnetmask,*vbroadcast;
 
 { struct sockaddr_in *sin;
   struct sockaddr_in netmask;
@@ -163,6 +165,8 @@ enum classes class;
   struct hostent *hp;
   struct in_addr inaddr;
 
+Verbose("Checking interface status...\n");
+  
 if ((hp = gethostbyname(VSYSNAME.nodename)) == NULL)
    {
    CfLog(cferror,"","gethostbyname");
@@ -174,7 +178,7 @@ else
    Verbose("Address given by nameserver: %s\n",inet_ntoa(inaddr));
    }
 
-strcpy(IFR.ifr_name,VIFDEV[class]);
+strcpy(IFR.ifr_name,vifdev);
 
 if (ioctl(sk,SIOCGIFADDR, (caddr_t) &IFR) == -1)   /* Get the device status flags */
    {
@@ -184,14 +188,6 @@ if (ioctl(sk,SIOCGIFADDR, (caddr_t) &IFR) == -1)   /* Get the device status flag
 sin = (struct sockaddr_in *) &IFR.ifr_addr;
 
 
-/*
-
-  How do we get the right byte ordering on systems with different host/net byte order?
-  
-Verbose("IP address on %s: %s\n",VIFDEV[class],inet_ntoa(sin->sin_addr));
-
-*/
-
 if (strcmp((char *)inet_ntoa(*(struct in_addr *)(hp->h_addr)),(char *)inet_ntoa(sin->sin_addr)) != 0)
    {
    CfLog(cferror,"This machine is configured with an address which differs from\n","");
@@ -207,11 +203,11 @@ if (ioctl(sk,SIOCGIFNETMASK, (caddr_t) &IFR) == -1)
 
 netmask.sin_addr = ((struct sockaddr_in *) &IFR.ifr_addr)->sin_addr;
 
-Verbose("Netmask: %s\n",inet_ntoa(netmask.sin_addr));
+Verbose("Found netmask: %s\n",inet_ntoa(netmask.sin_addr));
 
 strcpy(VBUFF,inet_ntoa(netmask.sin_addr));
 
-if (strcmp(VBUFF,VNETMASK))
+if (strcmp(VBUFF,vnetmask))
    {
    CfLog(cferror,"The netmask is incorrectly configured, resetting...\n","");
    insane = true;
@@ -225,13 +221,13 @@ if (ioctl(sk,SIOCGIFBRDADDR, (caddr_t) &IFR) == -1)
 sin = (struct sockaddr_in *) &IFR.ifr_addr;
 strcpy(VBUFF,inet_ntoa(sin->sin_addr));
 
-Verbose("Broadcast address: %s\n",inet_ntoa(sin->sin_addr));
+Verbose("Found broadcast address: %s\n",inet_ntoa(sin->sin_addr));
 
-GetBroadcastAddr(inet_ntoa(inaddr));
+GetBroadcastAddr(inet_ntoa(inaddr),vifdev,vnetmask,vbroadcast);
 
-if (strcmp(VBUFF,VBROADCAST) != 0)
+if (strcmp(VBUFF,VNUMBROADCAST) != 0)
    {
-   sprintf(OUTPUT,"Broadcast address was %s (should be %s)\n",VBUFF,VBROADCAST);
+   sprintf(OUTPUT,"Broadcast address was %s (should be %s)\n",VBUFF,vbroadcast);
    CfLog(cferror,OUTPUT,"");
    insane = true;
    }
@@ -241,10 +237,10 @@ return(insane);
 
 /*******************************************************************/
 
-SetIfStatus(sk,class)
+SetIfStatus(sk,vifdev,vnetmask,vbroadcast)
 
 int sk;
-enum classes class;
+char *vifdev,*vnetmask, *vbroadcast;
 
 { struct sockaddr_in *sin;
   struct sockaddr_in netmask, broadcast;
@@ -263,8 +259,9 @@ enum classes class;
 
 /* set netmask */
 
-strcpy(IFR.ifr_name,VIFDEV[class]);
-netmask.sin_addr.s_addr = inet_network(VNETMASK);
+Verbose("Resetting interface");
+strcpy(IFR.ifr_name,vifdev);
+netmask.sin_addr.s_addr = inet_network(vnetmask);
 IFR.ifr_addr = *((struct sockaddr *) &netmask);
 sin = (struct sockaddr_in *) &IFR.ifr_addr;
 
@@ -280,8 +277,8 @@ else
 
 /* broadcast addr */
 
-strcpy(IFR.ifr_name,VIFDEV[class]);
-broadcast.sin_addr.s_addr = inet_addr(VBROADCAST);
+strcpy(IFR.ifr_name,vifdev);
+broadcast.sin_addr.s_addr = inet_addr(vbroadcast);
 IFR.ifr_addr = *((struct sockaddr *) &broadcast);
 sin = (struct sockaddr_in *) &IFR.ifr_addr;
 
@@ -304,16 +301,16 @@ else
 
 /*****************************************************/
 
-GetBroadcastAddr(ipaddr)
+GetBroadcastAddr(ipaddr,vifdev,vnetmask,vbroadcast)
 
-char *ipaddr;
+char *ipaddr,*vifdev,*vnetmask,*vbroadcast;
 
 { unsigned int na,nb,nc,nd;
   unsigned int ia,ib,ic,id;
   unsigned int ba,bb,bc,bd;
   unsigned netmask,ip,broadcast;
 
-sscanf(VNETMASK,"%u.%u.%u.%u",&na,&nb,&nc,&nd);
+sscanf(vnetmask,"%u.%u.%u.%u",&na,&nb,&nc,&nd);
 
 netmask = nd + 256*nc + 256*256*nb + 256*256*256*na;
 
@@ -321,11 +318,11 @@ sscanf(ipaddr,"%u.%u.%u.%u",&ia,&ib,&ic,&id);
 
 ip = id + 256*ic + 256*256*ib + 256*256*256*ia;
 
-if (strcmp(VBROADCAST,"zero") == 0)
+if (strcmp(vbroadcast,"zero") == 0)
    {
    broadcast = ip & netmask;
    }
-else if (strcmp(VBROADCAST,"one") == 0)
+else if (strcmp(vbroadcast,"one") == 0)
    {
    broadcast = ip | (~netmask);
    }
@@ -338,7 +335,7 @@ ba = broadcast / (256 * 256 * 256);
 bb = (broadcast / (256 * 256)) % 256;
 bc = broadcast / (256) % 256;
 bd = broadcast % 256;
-sprintf(VBROADCAST,"%u.%u.%u.%u",ba,bb,bc,bd);
+sprintf(VNUMBROADCAST,"%u.%u.%u.%u",ba,bb,bc,bd);
 }
 
 /****************************************************************/
@@ -365,6 +362,8 @@ SetDefaultRoute()
 
   FILE *pp;
 
+Verbose("Looking at default route...\n");
+  
 if (getuid() != 0)                            
    {
    printf("%s: Only root can set a default route.\n",VPREFIX);
