@@ -79,7 +79,7 @@ AddSlash(newto);
 
 if (! MakeDirectoriesFor(newto))
    {
-   sprintf(OUTPUT,"Unable to make directory %s\n",from);
+   sprintf(OUTPUT,"Unable to make directory for %s in copy: %s to %s\n",newto,ip->path,ip->destination);
    CfLog(cferror,OUTPUT,"");
    return;
    }
@@ -136,7 +136,7 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
 
    strcat(newto,dirp->d_name);
 
-   if (TRAVLINKS)
+   if (TRAVLINKS || ip->linktype == 'n')
       {
       if (cfstat(newfrom,&statbuf,ip) == -1)
          {
@@ -149,16 +149,13 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       if (cflstat(newfrom,&statbuf,ip) == -1)
          {
          Verbose("%s: (Can't stat %s)\n",VPREFIX,newfrom);
-         if (cfreadlink(newfrom,VBUFF,bufsize,ip) != -1)
-            {
-            Verbose("          File is link to -> %s\n",VBUFF);
-            }
          continue;
          }
       }
 
    if (S_ISDIR(statbuf.st_mode))
       {	
+      bzero(&deststatbuf,sizeof(struct stat));
       save_uid = (ip->uid)->uid;
       save_gid = (ip->gid)->gid;
       
@@ -177,7 +174,7 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
 	 mkdir(newto,statbuf.st_mode);
 	 }
 
-      CheckCopiedFile(newto,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&statbuf,&statbuf,NULL,ip->acl_aliases);
+      CheckCopiedFile(newto,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&statbuf,NULL,ip->acl_aliases);
 
       (ip->uid)->uid = save_uid;
       (ip->gid)->gid = save_gid;
@@ -327,11 +324,20 @@ struct Image *ip;
   char destfile[bufsize];
   struct stat sourcestatbuf, deststatbuf;
   struct cfdirent *dirp;
-  int save_uid, save_gid;
+  int save_uid, save_gid, found;
   
 Debug2("CheckImage (source=%s destination=%s)\n",source,destination);
 
-if (cflstat(source,&sourcestatbuf,ip) == -1)
+if (ip->linktype == 'n')
+   {
+   found = cfstat(source,&sourcestatbuf,ip);
+   }
+else
+   {
+   found = cflstat(source,&sourcestatbuf,ip);
+   }
+ 
+if (found == -1)
    {
    sprintf(OUTPUT,"Can't stat %s\n",source);
    CfLog(cferror,OUTPUT,"");
@@ -366,8 +372,8 @@ if (S_ISDIR(sourcestatbuf.st_mode))
 
    if ((dirh = cfopendir(sourcedir,ip)) == NULL)
       {
-      printf("%s: Can't open directory %s\n",VPREFIX,sourcedir);
-      perror("opendir");
+      sprintf(OUTPUT,"%s: Can't open directory %s\n",VPREFIX,sourcedir);
+      CfLog(cferror,OUTPUT,"opendir");
       FlushClientCache(ip);
       (ip->uid)->uid = save_uid;
       (ip->gid)->gid = save_gid;
@@ -401,7 +407,7 @@ if (S_ISDIR(sourcestatbuf.st_mode))
 	 }
       
       strcat(destfile, dirp->d_name);
- 
+
       if (cflstat(sourcefile,&sourcestatbuf,ip) == -1)
          {
          printf("%s: Can't stat %s\n",VPREFIX,sourcefile);
@@ -438,6 +444,7 @@ struct Item *filelist;
 char *directory;
 
 { DIR *dirh;
+  struct stat statbuf; 
   struct dirent *dirp;
   char filename[bufsize];
 
@@ -474,13 +481,48 @@ Debug("PurgeFiles(%s)\n",directory);
 	  
 	  if (unlink(filename) == -1)
 	     {
-	     sprintf(OUTPUT,"Couldn't unlink %s\n",filename);
-	     CfLog(cfverbose,OUTPUT,"unlink");
-	     if (rmdir(filename) == -1)
+	     if (stat(filename,&statbuf) == -1)
 		{
-		sprintf(OUTPUT,"Couldn't unlink %s\n",filename);
-		CfLog(cfverbose,OUTPUT,"unlink");
+		sprintf(OUTPUT,"Couldn't stat %s while purging\n",filename);
+		CfLog(cfverbose,OUTPUT,"stat");
 		}
+
+	     if (S_ISDIR(statbuf.st_mode))
+		{
+    	        struct Tidy tp;
+		struct TidyPattern tpat;
+
+		tp.recurse;
+		tp.tidylist = &tpat;
+		tp.next = NULL;
+		tp.path = filename;
+
+		tpat.recurse = INFINITERECURSE;
+		tpat.age = 0;
+		tpat.size = 0;
+		tpat.pattern = strdup("*");
+		tpat.classes = strdup("any");;
+		tpat.defines = NULL;
+		tpat.dirlinks = 't';
+		tpat.rmdirs = 't';
+		tpat.searchtype = 'a';
+		tpat.log = 'd';
+		tpat.inform = 'd';
+		tpat.next = NULL;
+		RecursiveTidySpecialArea(filename,&tp,INFINITERECURSE);
+		free(tpat.pattern);
+		free(tpat.classes);
+		
+		if (rmdir(filename) == -1)
+		   {
+		   sprintf(OUTPUT,"Couldn't remove directory %s while purging\n",filename);
+		   CfLog(cfverbose,OUTPUT,"rmdir");
+		   }
+		continue;
+		}
+
+	     sprintf(OUTPUT,"Couldn't unlink %s while purging\n",filename);
+	     CfLog(cfverbose,OUTPUT,"");
 	     }
 	  }
        }
@@ -504,9 +546,9 @@ struct Image *ip;
 { char linkbuf[bufsize], *lastnode;
   struct stat deststatbuf;
   struct Link empty;
-  int i, silent = false;
+  int succeed, i, silent = false, enforcelinks;
   mode_t srcmode = sourcestatbuf.st_mode;
-  int ok_to_copy;
+  int ok_to_copy, found;
   
 Debug2("ImageCopy(%s,%s,+%o,-%o)\n",sourcefile,destfile,ip->plus,ip->minus);
 
@@ -517,25 +559,38 @@ if (IgnoredOrExcluded(image,sourcefile,ip->inclusions,ip->exclusions))
    return;
    }
 
-if (IsWildItemIn(VLINKCOPIES,lastnode) || IsWildItemIn(ip->symlink,lastnode))
+if (ip->linktype != 'n')
    {
-   Verbose("cfengine: copy item %s marked for linking instead\n",sourcefile);
-   switch (ip->linktype)
+   if (IsWildItemIn(VLINKCOPIES,lastnode) || IsWildItemIn(ip->symlink,lastnode))
       {
-      case 's':
-                LinkFiles(destfile,sourcefile,NULL,NULL,NULL,0,&empty);
-                break;
-      case 'r':
-                RelativeLink(destfile,sourcefile,NULL,NULL,NULL,0,&empty);
-	        break;
-      case 'a':
-                AbsoluteLink(destfile,sourcefile,NULL,NULL,NULL,0,&empty);
-                break;
-      default:
-                printf("%s: internal error, link type was [%c] in ImageCopy\n",VPREFIX,ip->linktype);
-                return;
+      Verbose("cfengine: copy item %s marked for linking instead\n",sourcefile);
+      enforcelinks = ENFORCELINKS;
+      ENFORCELINKS = true;
+      
+      switch (ip->linktype)
+	 {
+	 case 's':
+	     succeed = LinkFiles(destfile,sourcefile,NULL,NULL,NULL,0,&empty);
+	     break;
+	 case 'r':
+	     succeed = RelativeLink(destfile,sourcefile,NULL,NULL,NULL,0,&empty);
+	     break;
+	 case 'a':
+	     succeed = AbsoluteLink(destfile,sourcefile,NULL,NULL,NULL,0,&empty);
+	     break;
+	 default:
+	     printf("%s: internal error, link type was [%c] in ImageCopy\n",VPREFIX,ip->linktype);
+	     return;
+	 }
+
+      if (succeed)
+	 {
+	 ENFORCELINKS = enforcelinks;
+	 CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
+	 }
+      
+      return;
       }
-   return;
    }
 
 if (strcmp(ip->action,"silent") == 0)
@@ -545,7 +600,24 @@ if (strcmp(ip->action,"silent") == 0)
 
 bzero(linkbuf,bufsize);
 
-if (lstat(destfile,&deststatbuf) == -1)
+found = lstat(destfile,&deststatbuf);
+
+if (found != -1)
+   {
+   if (S_ISLNK(deststatbuf.st_mode) && (ip->linktype == 'n'))
+      {
+      if (unlink(destfile) == -1)
+	 {
+	 sprintf(OUTPUT,"Couldn't remove link",destfile);
+	 CfLog(cferror,OUTPUT,"unlink");
+	 return;
+	 }
+      Verbose("Removing old symbolic link %s to make way for copy\n",destfile);
+      found = -1;
+      }
+   }
+ 
+if (found == -1)
    {
    if (strcmp(ip->action,"warn") == 0)
       {
@@ -566,7 +638,7 @@ if (lstat(destfile,&deststatbuf) == -1)
       if (CopyReg(sourcefile,destfile,sourcestatbuf,deststatbuf,ip))
          {
 	 stat(destfile,&deststatbuf);
-         CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
+	 CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
 	 AddCopyClasses(ip);
          }
 
@@ -637,29 +709,35 @@ if (lstat(destfile,&deststatbuf) == -1)
          case 's':
 	           if (*linkbuf == '.')
 		      {
-		      RelativeLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
+		      succeed = RelativeLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
 		      }
 		   else
 		      {
-                      LinkFiles(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
+                      succeed = LinkFiles(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
 		      }
                    break;
          case 'r':
-                   RelativeLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
+                   succeed = RelativeLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
 	           break;
          case 'a':
-                   AbsoluteLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
+                   succeed = AbsoluteLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
                    break;
          default:
                    printf("cfengine: internal error, link type was [%c] in ImageCopy\n",ip->linktype);
                    return;
 	 }
 
-      AddCopyClasses(ip);
+      if (succeed)
+	 {
+	 CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
+	 AddCopyClasses(ip);
+	 }
       }
    }
 else
    {
+   Debug("Destination file %s exists\n",destfile);
+   
    if (! ip->force)
       {
       if (ip->size != cfnosize)
@@ -691,7 +769,7 @@ else
 	                  break;;
 	       }
          }
-   
+
       switch (ip->type)
          {
          case 'c': if (S_ISREG(deststatbuf.st_mode) && S_ISREG(srcmode))
@@ -746,7 +824,7 @@ else
 
 
 
-   if ((ip->force) || ok_to_copy)
+   if ((ip->force) || ok_to_copy || S_ISLNK(sourcestatbuf.st_mode))  /* Always check links */
       {
       if (S_ISREG(srcmode))
          {
@@ -758,7 +836,7 @@ else
          if (CopyReg(sourcefile,destfile,sourcestatbuf,deststatbuf,ip))
             {
             stat(destfile,&deststatbuf);
-            CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
+	    CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
             }
 
          return;
@@ -776,7 +854,37 @@ else
          sprintf(OUTPUT,"Checking link from %s to %s\n",destfile,linkbuf);
 	 CfLog(cfverbose,OUTPUT,"");
 
-         LinkFiles(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
+	 enforcelinks = ENFORCELINKS;
+	 ENFORCELINKS = true;
+
+	 switch (ip->linktype)
+	    {
+	    case 's':
+		if (*linkbuf == '.')
+		   {
+		   succeed = RelativeLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
+		   }
+		else
+		   {
+		   succeed = LinkFiles(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
+		   }
+		break;
+	    case 'r':
+		succeed = RelativeLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
+		break;
+	    case 'a':
+		succeed = AbsoluteLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
+		break;
+	    default:
+		printf("cfengine: internal error, link type was [%c] in ImageCopy\n",ip->linktype);
+		return;
+	    }
+
+	 if (succeed)
+	    {
+	    CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
+	    }
+	 ENFORCELINKS = enforcelinks;
          }
       }
    else
@@ -795,6 +903,8 @@ else
          return;
          }
 
+      CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
+
       Debug2("cfengine: image file is up to date: %s\n",destfile);
       }
    }
@@ -910,7 +1020,16 @@ if (strcmp(ip->server,"localhost") == 0)
    returnval->cf_list = NULL;
    returnval->cf_listpos = NULL;
    returnval->cf_dirh = opendir(name);
-   return returnval;
+
+   if (returnval->cf_dirh != NULL)
+      {
+      return returnval;
+      }
+   else
+      {
+      free ((char *)returnval);
+      return NULL;
+      }
    }
 else
    { 
@@ -973,6 +1092,7 @@ if ((dirh != NULL) && (dirh->cf_dirh != NULL))
 
 Debug("cfclosedir()\n");
 DeleteItemList(dirh->cf_list);
+free((char *)dirh); 
 }
 
 /*********************************************************************/
@@ -1063,8 +1183,13 @@ stat(new,&dstat);
 
 if ((dstat.st_size == 0) && (sstat.st_size != 0))
    {
-   sprintf(OUTPUT,"WARNING: new file was empty %s (old one wasn't)\n",new);
+   sprintf(OUTPUT,"WARNING: new file was empty %s (old one wasn't) aborting copy\n",new);
    CfLog(cfsilent,OUTPUT,"");
+   if (rename(backup,dest) == -1)
+      {
+      /* ignore */
+      }
+   return false;
    }
 
 if (rename(new,dest) == -1)
