@@ -62,7 +62,12 @@ Debug1("(Action is control, storing variable [%s=%s])\n",CURRENTITEM,value);
 switch (ScanVariable(CURRENTITEM))
    {
    case cfsite:
-   case cffaculty:  if (VFACULTY[0] != '\0')
+   case cffaculty:  if (!IsDefinedClass(CLASSBUFF))
+                      {
+		      break;
+		      }
+
+                    if (VFACULTY[0] != '\0')
                        {
                        yyerror("Multiple declaration of variable faculty / site");
                        FatalError("Redefinition of basic system variable");
@@ -71,7 +76,12 @@ switch (ScanVariable(CURRENTITEM))
                     strcpy(VFACULTY,value);
                     break;
 
-   case cfdomain:  if (strcmp(VDOMAIN,"undefined.domain") != 0)
+   case cfdomain:  if (!IsDefinedClass(CLASSBUFF))
+                      {
+		      break;
+		      }
+
+                   if (strcmp(VDOMAIN,"undefined.domain") != 0)
                       {
                       yyerror("Multiple declaration of variable domain");
                       FatalError("Redefinition of basic system variable");
@@ -79,9 +89,9 @@ switch (ScanVariable(CURRENTITEM))
 
                    strcpy(VDOMAIN,value);
 		  
-		   if (!strstr(VSYSNAME.nodename,VDOMAIN))
+		   if (!strstr(VSYSNAME.nodename,ToLowerStr(VDOMAIN)))
 		      {
-		      sprintf(VFQNAME,"%s.%s",VSYSNAME.nodename,VDOMAIN);
+		      sprintf(VFQNAME,"%s.%s",VSYSNAME.nodename,ToLowerStr(VDOMAIN));
 		      strcpy(VUQNAME,VSYSNAME.nodename);
 		      }
 		   else
@@ -110,11 +120,21 @@ switch (ScanVariable(CURRENTITEM))
                    break;
 
    case cfsysadm:  /* Can be redefined */
+                   if (!IsDefinedClass(CLASSBUFF))
+                      {
+		      break;
+		      }
 
                   strcpy(VSYSADM,value);
                   break;
 
-   case cfnetmask:  if (VNETMASK[0] != '\0')
+   case cfnetmask:
+                   if (!IsDefinedClass(CLASSBUFF))
+                      {
+		      break;
+		      }
+
+                  if (VNETMASK[0] != '\0')
                      {
                      yyerror("Multiple declaration of variable netmask");
                      FatalError("Redefinition of basic system variable");
@@ -136,13 +156,13 @@ switch (ScanVariable(CURRENTITEM))
                   break;
 
 
-   case cftimezone: if (VTIMEZONE[0] != '\0')
-                       {
-                       yyerror ("Multiple declaration of variable timezone");
-                       FatalError("Redefinition of basic system variable");
-                       }
+   case cftimezone:
+                   if (!IsDefinedClass(CLASSBUFF))
+                      {
+		      break;
+		      }
 
-                    strcpy(VTIMEZONE,value);
+                    AppendItem(&VTIMEZONE,value,NULL);
                     break;
 
    case cfssize: 
@@ -230,6 +250,10 @@ switch (ScanVariable(CURRENTITEM))
                   AddCompoundClass(value);
                   break;
 
+   case cfinstallclass:
+                  AddInstallable(value);
+		  break;
+		  
    case cfexcludecp:
                   PrependItem(&VEXCLUDECOPY,value,CLASSBUFF);
                   break;
@@ -302,8 +326,8 @@ switch (ScanVariable(CURRENTITEM))
 	     	        yyerror("Silly interface name, (should be something link eth0)");
 	                }
 
-                     strcpy(VIFNAME,value);
-		     VIFDEV[VSYSTEMHARDCLASS] = VIFNAME; /* override */
+                     strcpy(VIFNAMEOVERRIDE,value);
+		     VIFDEV[VSYSTEMHARDCLASS] = VIFNAMEOVERRIDE; /* override */
 		     Debug("Overriding interface with %s\n",VIFDEV[VSYSTEMHARDCLASS]);
 		     break;
 
@@ -381,6 +405,10 @@ switch(GetCommAttribute(item))
                    break;
    case cfdefine:  HandleDefine(value);
                    break;
+   case cfsetlog:  HandleSetLog(value);
+                   break;
+   case cfsetinform: HandleSetInform(value);
+                   break;
    default:        yyerror("Illegal file attribute");
    }
 }
@@ -445,6 +473,10 @@ switch(GetCommAttribute(item))
                    break;
    case cfpurge:   HandlePurge(value);
                    break;
+   case cfsetlog:  HandleSetLog(value);
+                   break;
+   case cfsetinform: HandleSetInform(value);
+                   break;
 
    default:        yyerror("Illegal copy attribute");
    }
@@ -477,6 +509,11 @@ switch(GetCommAttribute(item))
                    break;
    case cfdefine:  HandleDefine(value);
                    break;
+   case cfsetlog:  HandleSetLog(value);
+                   break;
+   case cfsetinform: HandleSetInform(value);
+                   break;
+
    default:        yyerror("Illegal disk/required attribute");
    }
 
@@ -484,6 +521,80 @@ switch(GetCommAttribute(item))
 
 /******************************************************************/
 
+HandleOptionalInterface(item)
+
+char *item;
+
+{ char value[maxvarsize];
+
+VBUFF[0] = value[0] = '\0';
+
+ExpandVarstring(item,VBUFF,NULL);
+
+sscanf(VBUFF,"%*[^=]=%s",value);
+
+if (value[0] == '\0')
+   {
+   yyerror("intefaces attribute with no value");
+   }
+
+Debug1("HandleOptionalRequiredAttribute(%s)\n",value);
+
+switch(GetCommAttribute(item))
+   {
+   case cfsetnetmask:   HandleNetmask(value);
+                        break;
+   case cfsetbroadcast: HandleBroadcast(value);
+                        break;
+
+   default:        yyerror("Illegal interfaces attribute");
+   }
+
+}
+
+
+/******************************************************************/
+
+HandleOptionalMiscMountsItem(item)
+
+char *item;
+
+{ char value[maxvarsize];
+
+VBUFF[0] = value[0] = '\0';
+
+ExpandVarstring(item,VBUFF,NULL);
+
+sscanf(VBUFF,"%*[^=]=%s",value);
+
+if (value[0] == '\0')
+   {
+   yyerror("Miscmounts attribute with no value");
+   }
+
+Debug1("HandleOptionalMiscMOuntsItem(%s)\n",value);
+
+switch(GetCommAttribute(item))
+   {
+   case cfmode:    if (strcmp(value,"rw") == 0)
+                      {
+		      MOUNTMODE='w';
+		      break;
+                      }
+   
+                   if (strcmp(value,"ro") == 0)
+                      {
+		      MOUNTMODE='o';
+		      break;
+                      }
+   
+   default:        yyerror("Illegal miscmounts attribute (rw/ro)");
+   }
+
+}
+
+/******************************************************************/
+
 HandleOptionalTidyAttribute(item)
 
 char *item;
@@ -536,6 +647,10 @@ switch(GetCommAttribute(item))
 
    case cfdefine:  HandleDefine(value);
                    break;
+   case cfsetlog:  HandleSetLog(value);
+                   break;
+   case cfsetinform: HandleSetInform(value);
+                   break;
 
    default:        yyerror("Illegal tidy attribute");
    }
@@ -572,6 +687,11 @@ switch(GetCommAttribute(item))
                    break;
    case cfdefine:  HandleDefine(value);
                    break;
+   case cfsetlog:  HandleSetLog(value);
+                   break;
+   case cfsetinform: HandleSetInform(value);
+                   break;
+
    default:        yyerror("Illegal directory attribute");
    }
 }
@@ -610,6 +730,10 @@ switch(GetCommAttribute(item))
                    break;
    case cfdefine:  HandleDefine(value);
                    break;
+   case cfsetlog:  HandleSetLog(value);
+                   break;
+   case cfsetinform: HandleSetInform(value);
+                   break;
 
    default:        yyerror("Illegal disable attribute");
    }
@@ -657,6 +781,11 @@ switch(GetCommAttribute(item))
                    break;
    case cfdefine:  HandleDefine(value);
                    break;
+   case cfsetlog:  HandleSetLog(value);
+                   break;
+   case cfsetinform: HandleSetInform(value);
+                   break;
+
    default:        yyerror("Illegal link attribute");
    }
 }
@@ -710,6 +839,14 @@ switch(GetCommAttribute(item))
                    break;
    case cfuseshell:HandleUseShell(value);
                    break;
+   case cfsetlog:  HandleSetLog(value);
+                   break;
+   case cfsetinform: HandleSetInform(value);
+                   break;
+   case cfexclude: PrependItem(&VEXCLUDEPARSE,value,"any");
+                   break;
+   case cfinclude: PrependItem(&VINCLUDEPARSE,value,"any");
+                   break;
    default:        yyerror("Illegal process attribute");
    }
 }
@@ -741,6 +878,11 @@ switch(GetCommAttribute(item))
                     break;
    case cfuseshell: HandleUseShell(value);
                     break;
+   case cfsetlog:  HandleSetLog(value);
+                   break;
+   case cfsetinform: HandleSetInform(value);
+                   break;
+
    default:         yyerror("Illegal shellcommand attribute");
    }
 
@@ -871,6 +1013,12 @@ switch (type)
                       break;
                       }
 
+                   if (strcmp(item,VFQNAME) == 0)
+                      {
+                      AddClassToHeap(GROUPBUFF);
+                      break;
+                      }
+
                    if (IsItemIn(VHEAP,item))  /* group reference */
                       {
                       AddClassToHeap(GROUPBUFF);
@@ -1069,72 +1217,6 @@ Debug1("Installing mailserver (%s) for group (%s)",path,GROUPBUFF);
 
 /*******************************************************************/
 
-AppendScript(item,timeout,useshell)
-
-char *item, useshell;
-int timeout;
-
-{ struct TwoDimList *tp = NULL;
-  struct ShellComm *ptr;
-  char *sp;
-
-Debug1("Installing item (%s) in the script list\n",item);
-
-if ( ! IsInstallable(CLASSBUFF))
-   {
-   Debug1("Not installing %s, no match\n",item);
-   return;
-   }
-
-Build2DListFromVarstring(&tp,item,' '); /* Must be at least one space between each var */
-
-Set2DList(tp);
-
-for (sp = Get2DListEnt(tp); sp != NULL; sp = Get2DListEnt(tp))
-   {
-   if (*sp != '/')
-      {
-      yyerror("scripts or commands must have absolute path names");
-      printf ("cfengine: concerns: %s\n",sp);
-      return;
-      }
-
-   if ((ptr = (struct ShellComm *)malloc(sizeof(struct ShellComm))) == NULL)
-      {
-      FatalError("Memory Allocation failed for AppendScript() #1");
-      }
-
-   if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
-      {
-      FatalError("Memory Allocation failed for Appendscript() #2");
-      }
-
-   if ((ptr->name = strdup(sp)) == NULL)
-      {
-      FatalError("Memory Allocation failed for Appendscript() #3");
-      }
-
-   if (VSCRIPTTOP == NULL)                 /* First element in the list */
-      {
-      VSCRIPT = ptr;
-      }
-   else
-      {
-      VSCRIPTTOP->next = ptr;
-      }
-
-   ptr->timeout = timeout;
-   ptr->useshell = useshell;
-   ptr->next = NULL;
-   
-   VSCRIPTTOP = ptr;
-   }
-
-Delete2DList(tp);
-}
-
-/*******************************************************************/
-
 InstallLinkItem (from,to)
 
 char *from, *to;
@@ -1146,16 +1228,26 @@ Debug1("Storing Link: (From)%s->(To)%s\n",from,to);
 
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing link no match\n");
    return;
    }
 
 bzero(VBUFF,bufsize);
 ExpandVarstring(from,VBUFF,"");
+if (strlen(VBUFF) > 1)
+   {
+   DeleteSlash(VBUFF);
+   }
 
 bzero(buffer,bufsize);
 ExpandVarstring(to,buffer,"");
 
+if (strlen(VBUFF) > 1)
+   { 
+   DeleteSlash(buffer);
+   }
+
 bzero(buffer2,bufsize);
 ExpandVarstring(ALLCLASSBUFFER,buffer2,""); 
  
@@ -1215,7 +1307,10 @@ ptr->exclusions = VEXCLUDEPARSE;
 ptr->inclusions = VINCLUDEPARSE;
 ptr->recurse = VRECURSE;
 ptr->nofile = DEADLINKS;
- 
+ptr->log = LOGP;
+ptr->inform = INFORMP;
+
+
 VLINKTOP = ptr;
 
 if (ptr->recurse != 0)
@@ -1240,6 +1335,7 @@ Debug1("Storing Linkchildren item: %s -> %s\n",from,to);
 
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing linkchildren no match\n");
    return;
    }
@@ -1300,6 +1396,9 @@ AddInstallable(ptr->defines);
    ptr->exclusions = VEXCLUDEPARSE;
    ptr->inclusions = VINCLUDEPARSE;
    ptr->recurse = VRECURSE;
+   ptr->log = LOGP;
+   ptr->inform = INFORMP;
+
    VCHLINKTOP = ptr;
 
    if (ptr->recurse != 0 && strcmp(ptr->to,"linkchildren") == 0)
@@ -1328,6 +1427,7 @@ Debug1("Installing item (%s) in the required list\n",path);
 
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing %s, no match\n",path);
    return;
    }
@@ -1373,6 +1473,9 @@ else
 AddInstallable(ptr->define);
 ptr->freespace = freespace;
 ptr->next = NULL;
+ptr->log = LOGP;
+ptr->inform = INFORMP;
+
 VREQUIREDTOP = ptr;
 
 InitializeAction();
@@ -1399,6 +1502,7 @@ char *path;
 { 
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing %s, no match\n",path);
    return;
    }
@@ -1544,10 +1648,30 @@ switch (action)
                   AppendScript(CURRENTPATH,VTIMEOUT,USESHELL);
 		  break;
 
+   case interfaces:
+                  AppendInterface(VIFNAME,DESTINATION,CURRENTPATH);
+		  break;
+
    case required:
                   InstallRequiredPath(CURRENTPATH,ALLCLASSBUFFER,IMGSIZE);
                   break;
 
+   case misc_mounts:
+                  if ((strlen(MOUNTFROM) != 0) && (strlen(MOUNTONTO) != 0))
+		     {
+		     switch (MOUNTMODE)
+			{
+			case 'o': AppendMiscMount(MOUNTFROM,MOUNTONTO,"ro");
+			    break;
+			case 'w': AppendMiscMount(MOUNTFROM,MOUNTONTO,"rw");
+			    break;
+			default:  printf("Install pending, miscmount, shouldn't happen\n");
+			}
+		     }
+		  
+		  InitializeAction();
+		  break;
+		   
    case links:
 
                   if (LINKTO[0] == '\0')
@@ -1625,6 +1749,7 @@ else
 
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing Edit no match\n");
    return;
    }
@@ -1833,6 +1958,147 @@ return (NoEdit);
 }
 
 
+/********************************************************************/
+
+AppendInterface(ifname,netmask,broadcast)
+
+char *ifname,*netmask,*broadcast;
+
+{ struct Interface *ifp;
+ 
+Debug1("Installing item (%s:%s:%s) in the interfaces list\n",ifname,netmask,broadcast);
+
+if (!IsInstallable(CLASSBUFF))
+   {
+   InitializeAction();
+   Debug1("Not installing %s, no match\n",ifname);
+   return;
+   }
+
+if (strlen(netmask) < 7)
+   {
+   yyerror("illegal or missing netmask");
+   InitializeAction();
+   return;
+   }
+
+if (strlen(broadcast) < 3)
+   {
+   yyerror("illegal or missing broadcast address");
+   InitializeAction();
+   return;
+   }
+ 
+if ((ifp = (struct Interface *)malloc(sizeof(struct Interface))) == NULL)
+   {
+   FatalError("Memory Allocation failed for AppendInterface() #1");
+   }
+
+if ((ifp->classes = strdup(CLASSBUFF)) == NULL)
+   {
+   FatalError("Memory Allocation failed for Appendinterface() #2");
+   }
+
+if ((ifp->ifdev = strdup(ifname)) == NULL)
+   {
+   FatalError("Memory Allocation failed for AppendInterface() #3");
+   }
+ 
+if ((ifp->netmask = strdup(netmask)) == NULL)
+   {
+   FatalError("Memory Allocation failed for AppendInterface() #3");
+   }
+
+if ((ifp->broadcast = strdup(broadcast)) == NULL)
+   {
+   FatalError("Memory Allocation failed for AppendInterface() #3");
+   } 
+
+if (VIFLISTTOP == NULL)                 /* First element in the list */
+   {
+   VIFLIST = ifp;
+   }
+ else
+    {
+    VIFLISTTOP->next = ifp;
+    }
+
+ifp->next = NULL;
+   
+VIFLISTTOP = ifp;
+ 
+InitializeAction(); 
+}
+
+/*******************************************************************/
+
+AppendScript(item,timeout,useshell)
+
+char *item, useshell;
+int timeout;
+
+{ struct TwoDimList *tp = NULL;
+  struct ShellComm *ptr;
+  char *sp;
+
+Debug1("Installing item (%s) in the script list\n",item);
+
+if ( ! IsInstallable(CLASSBUFF))
+   {
+   InitializeAction();
+   Debug1("Not installing %s, no match\n",item);
+   return;
+   }
+
+Build2DListFromVarstring(&tp,item,' '); /* Must be at least one space between each var */
+
+Set2DList(tp);
+
+for (sp = Get2DListEnt(tp); sp != NULL; sp = Get2DListEnt(tp))
+   {
+   if (*sp != '/')
+      {
+      yyerror("scripts or commands must have absolute path names");
+      printf ("cfengine: concerns: %s\n",sp);
+      return;
+      }
+
+   if ((ptr = (struct ShellComm *)malloc(sizeof(struct ShellComm))) == NULL)
+      {
+      FatalError("Memory Allocation failed for AppendScript() #1");
+      }
+
+   if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
+      {
+      FatalError("Memory Allocation failed for Appendscript() #2");
+      }
+
+   if ((ptr->name = strdup(sp)) == NULL)
+      {
+      FatalError("Memory Allocation failed for Appendscript() #3");
+      }
+
+   if (VSCRIPTTOP == NULL)                 /* First element in the list */
+      {
+      VSCRIPT = ptr;
+      }
+   else
+      {
+      VSCRIPTTOP->next = ptr;
+      }
+
+   ptr->log = LOGP;
+   ptr->inform = INFORMP;
+   ptr->timeout = timeout;
+   ptr->useshell = useshell;
+   ptr->next = NULL;
+   
+   VSCRIPTTOP = ptr;
+   }
+
+Delete2DList(tp);
+}
+
 /********************************************************************/
 
 AppendDisable(path,type,rotate,comp,size,defines)
@@ -1848,6 +2114,7 @@ Debug1("Installing item (%s) in the disable list\n",path);
 
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing %s, no match\n",path);
    return;
    }
@@ -1917,6 +2184,9 @@ ptr->rotate = rotate;
 ptr->comp = comp;
 ptr->size = size;
 ptr->next = NULL;
+ptr->log = LOGP;
+ptr->inform = INFORMP;
+
 VDISABLETOP = ptr;
 InitializeAction();
 AddInstallable(ptr->defines); 
@@ -1928,13 +2198,16 @@ InstallTidyItem (path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classe
 
 char *wild, *path;
 short age,tidysize,tidydirs;
-int rec, travlinks;
-char type, ldirs, *classes, *defines;
+int rec;
+char type, ldirs, *classes, *defines, travlinks;
 
 { struct TwoDimList *tp = NULL;
   char *sp;
 
-DeleteSlash(path);
+if (strcmp(path,"/") != 0)
+   {
+   DeleteSlash(path);
+   }
   
 Build2DListFromVarstring(&tp,path,'/');
    
@@ -1972,6 +2245,7 @@ Debug1("InstallMakePath (%s) (+%o)(-%o)(%s)(%s)\n",path,plus,minus,uidnames,gidn
 
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing directory item, no match\n");
    return;
    }
@@ -2022,6 +2296,9 @@ ptr->gid = MakeGidList(gidnames);
 ptr->inclusions = NULL;
 ptr->exclusions = NULL;
 ptr->acl_aliases= VACLBUILD; 
+ptr->log = LOGP;
+ptr->inform = INFORMP;
+
 ptr->next = NULL;
 VMAKEPATHTOP = ptr;
 InitializeAction();
@@ -2406,13 +2683,19 @@ char *value;
 {
  if (strcmp(value,"true") == 0)
    {
-   USESHELL = true;
+   USESHELL = 'y';
    return;
    }
 
 if (strcmp(value,"false") == 0)
    {
-   USESHELL = false;
+   USESHELL = 'n';
+   return;
+   }
+
+if (strcmp(value,"dumb") == 0)
+   {
+   USESHELL = 'd';
    return;
    }
 
@@ -2421,6 +2704,50 @@ yyerror("Illegal attribute for useshell= ");
 
 /*******************************************************************/
 
+HandleSetLog(value)
+
+char *value;
+
+{
+if (strcmp(value,"true") == 0 || strcmp(value,"on") == 0)
+   {
+   LOGP = 't';
+   return;
+   }
+
+if (strcmp(value,"false") == 0 || strcmp(value,"off") == 0)
+   {
+   LOGP = 'f';
+   return;
+   }
+
+yyerror("Illegal attribute for log= ");
+}
+
+/*******************************************************************/
+
+HandleSetInform(value)
+
+char *value;
+
+{
+if (strcmp(value,"true") == 0 || strcmp(value,"on") == 0)
+   {
+   INFORMP = 't';
+   return;
+   }
+
+if (strcmp(value,"false") == 0 || strcmp(value,"off") == 0)
+   {
+   INFORMP = 'f';
+   return;
+   }
+
+yyerror("Illegal attribute for inform= ");
+}
+
+/*******************************************************************/
+
 GetFileAction(action)
 
 char *action;
@@ -2460,6 +2787,7 @@ Debug1("InstallFileaction (%s) (+%o)(-%o) (%s) (%d) (%c)\n",path,plus,minus,FILE
 
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing file item, no match\n");
    return;
    }
@@ -2515,6 +2843,9 @@ for (spl = Get2DListEnt(tp); spl != NULL; spl = Get2DListEnt(tp))
    ptr->travlinks = travlinks;
    ptr->acl_aliases  = VACLBUILD;
    ptr->next = NULL;
+   ptr->log = LOGP;
+   ptr->inform = INFORMP;
+
    VFILETOP = ptr;
    }
 
@@ -2552,6 +2883,7 @@ ExpandVarstring(defines,buf3,"");
 
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing process item, no match\n");
    return;
    }
@@ -2601,6 +2933,10 @@ ptr->signal = signal;
 ptr->action = action;
 ptr->useshell = useshell; 
 ptr->next = NULL;
+ptr->log = LOGP;
+ptr->inform = INFORMP;
+ptr->exclusions = VEXCLUDEPARSE;
+ptr->inclusions = VINCLUDEPARSE;
 VPROCTOP = ptr;
 InitializeAction();
 AddInstallable(ptr->defines); 
@@ -2629,6 +2965,7 @@ int rec, size;
   
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing copy item, no match\n");
    return;
    }
@@ -2770,6 +3107,9 @@ for (spl = Get2DListEnt(tp); spl != NULL; spl = Get2DListEnt(tp))
    ptr->inclusions = VINCLUDEPARSE;
    ptr->cache = NULL;
    ptr->purge = purge;
+   ptr->log = LOGP;
+   ptr->inform = INFORMP;
+
    ptr->acl_aliases = VACLBUILD;
    
    if (strcmp(buf3,"localhost") != 0)
@@ -3066,6 +3406,56 @@ PROSIGNAL = (short) i;
 
 /*******************************************************************/
 
+HandleNetmask(value)
+
+char *value;
+
+{
+ if (strlen(DESTINATION) == 0)
+    {
+    strcpy(DESTINATION,value);
+    }
+ else
+    {
+    yyerror("redefinition of netmask");
+    }
+}
+
+/*******************************************************************/
+
+HandleBroadcast(value)
+
+char *value;
+
+{
+if (strlen(CURRENTPATH) != 0)
+    {
+    yyerror("redefinition of broadcast address");
+    }
+
+if (strcmp("ones",value) == 0)
+   {
+   strcpy(CURRENTPATH,"one");
+   return;
+   }
+
+if (strcmp("zeroes",value) == 0)
+   {
+   strcpy(CURRENTPATH,"zero");
+   return;
+   }
+
+if (strcmp("zeros",value) == 0)
+   {
+   strcpy(CURRENTPATH,"zero");
+   return;
+   }
+
+yyerror("Illegal broadcast specifier (ones/zeros)"); 
+}
+
+/*******************************************************************/
+
 AppendToActionSequence (action)
 
 char *action;
@@ -3168,13 +3558,13 @@ char *value;
 {
 if (strcmp(value,"kill") == 0)
    {
-   DEADLINKS = true;
+   DEADLINKS = false;
    return;
    }
 
 if (strcmp(value,"force") == 0)
    {
-   DEADLINKS = false;
+   DEADLINKS = true;
    return;
    }
  
@@ -3221,7 +3611,15 @@ if (strcmp(value,"rel") == 0 || strcmp(value,"relative") == 0)
    LINKTYPE = 'r';
    return;
    }
-yyerror("Invalid link type");
+ 
+if (strcmp(value,"none") == 0 || strcmp(value,"copy") == 0)
+   {
+   LINKTYPE = 'n';
+   return;
+   }
+
+ sprintf(OUTPUT,"Invalid link type %s\n",value);
+yyerror(OUTPUT);
 }
 
 /*******************************************************************/
@@ -3282,13 +3680,16 @@ char *uidnames;
   char *sp;
   int offset;
   struct passwd *pw;
-  char *machine, *user, *domain;
+  char *machine, *user, *domain, buffer[bufsize];
   int uid;
   int tmp;
 
 uidlist = NULL;
-
-for (sp = uidnames; *sp != '\0'; sp+=strlen(uidbuff))
+buffer[0] = '\0';
+ 
+ExpandVarstring(uidnames,buffer,"");
+ 
+for (sp = buffer; *sp != '\0'; sp+=strlen(uidbuff))
    {
    if (*sp == ',')
       {
@@ -3383,7 +3784,7 @@ struct GidList *MakeGidList(gidnames)
 char *gidnames;
 
 { struct GidList *gidlist;
-  char gidbuff[bufsize];
+  char gidbuff[bufsize],buffer[bufsize];
   char err[bufsize];
   char *sp;
   struct group *gr;
@@ -3391,8 +3792,11 @@ char *gidnames;
   int tmp;
 
 gidlist = NULL;
-
-for (sp = gidnames; *sp != '\0'; sp+=strlen(gidbuff))
+buffer[0] = '\0';
+ 
+ExpandVarstring(gidnames,buffer,"");
+ 
+for (sp = buffer; *sp != '\0'; sp+=strlen(gidbuff))
    {
    if (*sp == ',')
       {
@@ -3442,8 +3846,8 @@ InstallTidyPath(path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes
 
 char *wild, *path, *defines;
 short age,tidydirs;
-int rec, travlinks,tidysize;
-char type, ldirs, *classes;
+int rec, tidysize;
+char type, ldirs, *classes, travlinks;
 
 { struct Tidy *ptr;
   char *sp;
@@ -3451,6 +3855,7 @@ char type, ldirs, *classes;
 
 if (!IsInstallable(classes))
    {
+   InitializeAction();
    Debug1("Not installing tidy path, no match\n");
    return;
    }
@@ -3496,6 +3901,7 @@ if (rec != INFINITERECURSE && strncmp("home/",ptr->path,5) == 0) /* Is this a su
 ptr->tidylist = NULL;
 ptr->recurse = rec + no_of_links;
 ptr->next = NULL;
+
 VTIDYTOP = ptr;
 
 AddTidyItem(path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,defines);
@@ -3509,8 +3915,8 @@ AddTidyItem(path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,def
 
 char *wild, *path, *defines;
 short age,tidysize,tidydirs;
-int rec, travlinks;
-char type, ldirs,*classes;
+int rec;
+char type, ldirs,*classes, travlinks;
 
 { char varbuff[bufsize];
   struct Tidy *ptr;
@@ -3519,6 +3925,7 @@ Debug1("AddTidyItem(%s,pat=%s)\n",path,wild);
 
 if ( ! IsInstallable(CLASSBUFF))
    {
+   InitializeAction();
    Debug1("Not installing TidyItem no match\n");
    return;
    }
@@ -3751,8 +4158,8 @@ PrependTidy(list,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,def
 struct TidyPattern **list;
 char *wild;
 short age,tidysize,tidydirs;
-int rec, travlinks;
-char type, ldirs, *classes, *defines;
+int rec;
+char type, ldirs, *classes, *defines, travlinks;
 
 { struct TidyPattern *tp;
   char *spe,*sp, buffer[bufsize];
@@ -3784,6 +4191,11 @@ if ((classes!= NULL) && (spe = malloc(strlen(classes)+2)) == NULL)
    FatalError("");
    }
 
+if (travlinks == '?')
+   {
+   travlinks = 'F';  /* False is default */
+   }
+
 tp->size = tidysize;
 tp->recurse = rec;
 tp->age = age;
@@ -3792,6 +4204,8 @@ tp->travlinks = travlinks;
 tp->pattern = sp;
 tp->next = *list;
 tp->dirlinks = ldirs;
+tp->log = LOGP;
+tp->inform = INFORMP;
 
 switch (tidydirs)
    {
