@@ -659,6 +659,8 @@ char *string;
 
 { struct Item *ip, *next, *last;
 
+Debug("DeleteItemContaining(%s)\n",string);
+ 
 for (ip = *list; ip != NULL; ip=ip->next)
    {
    if (EDABORTMODE && ItemMatchesRegEx(ip->name,VEDITABORT))
@@ -1377,8 +1379,10 @@ for (ip = *liststart; ip != NULL; ip=ip->next)
 
    CURRENTLINEPTR = ip;                  /* patch */
    InsertItemAfter(liststart,ip,VBUFF);
+   CURRENTLINEPTR = ip->next;
    DeleteItem(liststart,ip);
-   /* ip = ip->next; */                  /* patch */
+
+   ip = CURRENTLINEPTR->next;
    }
 
 CURRENTLINEPTR = oldCurrentLinePtr;      /* patch */
