@@ -82,7 +82,7 @@ while(!feof(fp))
    {
    ReadLine(VBUFF,bufsize,fp);
 
-   if (!feof(fp) || strlen(VBUFF) != 0) 
+   if (!feof(fp) || (strlen(VBUFF) != 0))
       {
       AppendItem(liststart,VBUFF,NULL);
       }
