@@ -545,6 +545,8 @@ if (readlink(from,linkbuf,bufsize) == -1)
    }
 else
    { int off1 = 0, off2 = 0;
+
+   DeleteSlash(linkbuf);
    
    if (strncmp(linkbuf,"./",2) == 0)   /* Ignore ./ at beginning */
       {
@@ -571,8 +573,7 @@ else
                return true;
                }
 
-            DoLink(from,to,ptr->defines);
-            return true;
+            return DoLink(from,to,ptr->defines);
             }
          }
       else
@@ -597,8 +598,7 @@ else
       }
    }
 
-DoLink(from,to,ptr->defines);
-return(true);
+return DoLink(from,to,ptr->defines);
 }
 
 /*********************************************************************/
@@ -747,20 +747,23 @@ char *from, *to, *defines;
 {
 if (DONTDO)
    {
-   printf("cfengine: Need to link files %s -> %s\n\n",from,to);
+   printf("cfengine: Need to link files %s -> %s\n",from,to);
    }
 else
    {
-   sprintf(OUTPUT,"Linking files %s -> %s\n\n",from,to);
+   sprintf(OUTPUT,"Linking files %s -> %s\n",from,to);
    CfLog(cfsilent,OUTPUT,"");
 
    if (symlink(to,from) == -1)
       {
-      perror("symlink");
+      sprintf(OUTPUT,"Couldn't link %s to %s\n",to,from);
+      CfLog(cferror,OUTPUT,"symlink");
+      return false;
       }
    else
       {
       AddMultipleClasses(defines);
+      return true;
       }
    }
 }
@@ -1148,7 +1151,15 @@ while (pop > 0)
     pop--;
     }
 
-AddSlash(CURRENTPATH);
+if (strlen(CURRENTPATH) == 0)
+   {
+   strcpy(CURRENTPATH,"/");
+   }
+else
+   {
+   AddSlash(CURRENTPATH);
+   }
+ 
 strcat(CURRENTPATH,sp);
 Debug("Reconstructed absolute linkname = %s\n",CURRENTPATH);
 return CURRENTPATH; 
