@@ -146,3 +146,41 @@ if ((string == NULL) || (strlen(string) == 0))
     printf("\n");
     }
 }
+
+/*****************************************************************************/
+
+ResetOutputRoute (log,inform)
+
+char log, inform;
+
+  /* t = true, f = false, d = default */
+
+{
+if ((log == 't') || (log == 'f') || (inform == 't') || (inform == 'f'))
+   {
+   INFORM_save = INFORM;
+   LOGGING_save = LOGGING;
+   
+   switch (log)
+      {
+      case 't': LOGGING = true;
+	        break;
+      case 'f': LOGGING = false;
+	        break;
+      }
+
+   switch (inform)
+      {
+      case 't': INFORM = true;
+	        break;
+      case 'f': INFORM = false;
+	        break;
+      }
+   }
+else
+   {
+   INFORM = INFORM_save;
+   LOGGING = LOGGING_save;
+   }
+}
+
