@@ -100,6 +100,7 @@ if ((sp = malloc(strlen(VSYSNAME.nodename)+1)) == NULL)
    }
 
 strcpy(sp,VSYSNAME.nodename);
+SetDomainName(sp);
  
 strcpy(VPREFIX,VSYSNAME.nodename);
 
@@ -108,18 +109,12 @@ for (sp2=sp; *sp2 != '\0'; sp2++)  /* Truncate fully qualified name */
    if (*sp2 == '.')
       {
       *sp2 = '\0';
-      Debug("cfengine: INFO! Truncating fully qualified hostname %s to %s\n",VSYSNAME.nodename,sp);
+      Debug("Truncating fully qualified hostname %s to %s\n",VSYSNAME.nodename,sp);
       break;
       }
-
-   if (!isalnum(*sp2))
-      {
-      *sp2 = '_';
-      }
    }
 
-AddClassToHeap(CanonifyName(sp));                 /* Unqualified host name comes first */
-VDEFAULTBINSERVER.name = sp;
+AddClassToHeap(CanonifyName(sp));
 
 VSYSTEMHARDCLASS = (enum classes) i;
 
@@ -347,3 +342,33 @@ Verbose("%s\n",string);
 Verbose("---------------------------------------------------------------------\n\n");
 }
 
+
+/*********************************************************************/
+
+SetDomainName(sp)           /* Bas van der Vlies */
+
+char *sp;
+
+{ char fqn[maxvarsize];
+  char *ptr;
+  char buffer[bufsize];
+
+if (gethostname(fqn, sizeof(fqn)) != -1 )
+   {
+   if (strcmp(fqn, sp) != 0)
+      {
+      strcpy(VFQNAME,fqn);
+      strcpy(buffer,VFQNAME);
+      AddClassToHeap(CanonifyName(buffer));
+      AddClassToHeap(CanonifyName(ToLowerStr(buffer)));
+      
+      ptr = strchr(fqn, '.');
+      strcpy(VDOMAIN, ++ptr);
+      VDEFAULTBINSERVER.name = VFQNAME;
+      }
+   else
+      {
+      VDEFAULTBINSERVER.name = sp;
+      }
+   }
+}
