@@ -91,12 +91,32 @@ return(false);
 
 IsMountedFileSystem (childstat,dir,rlevel)
 
+ /* Is FS NFS mounted ? */
+
 char *dir;
 struct stat *childstat;
 int rlevel;
 
 { struct stat parentstat;
+  struct Item *ip;
+  char host[maxvarsize];
 
+for (ip = VMOUNTABLES; ip !=NULL; ip=ip->next)
+   {
+   if (strstr(ip->name,dir))
+      {
+      sscanf(ip->name,"%[^:]",host);
+
+      Debug("Looking at filesystem %s on %s\n",ip->name,host);
+
+      if (strncmp(host,VFQNAME,strlen(host)) == 0)
+	 {
+	 Verbose("Filesystem %s belongs to this host\n",dir);
+	 return false;
+	 }
+      }
+   }
+  
 strcpy(VBUFF,dir);
 
 if (VBUFF[strlen(VBUFF)-1] == '/')
