@@ -472,19 +472,35 @@ else
                         FatalError("Fatal Error");
       case broadcast:   InstallBroadcastItem(item);
                         break;
+			
+      case interfaces:  if (strlen(VIFNAME)==0)
+	                   {			   
+			   strcpy(VIFNAME,item);
+			   }
+                        else
+			   {
+			   InstallPending(ACTION);
+			   strcpy(VIFNAME,item);
+			   }
+	                break;
       case defaultroute:InstallDefaultRouteItem(item);
                         break;
       case misc_mounts: if (MOUNT_FROM && MOUNT_ONTO && (strcmp(item,"rw") == 0 || strcmp(item,"ro") == 0))
                            {
-                           AppendMiscMount(MOUNTFROM,MOUNTONTO,item);
-                           CURRENTITEM[0] = '\0';
-                           MOUNT_FROM = false;
-                           MOUNT_ONTO = false;
-                           }
-                        else
-                           {
-                           yyerror("miscmounts: host:/frompath /mounton_path ro|rw\n");
-                           }
+			   if (strcmp(item,"rw") == 0)
+			      {
+			      MOUNTMODE='w';
+			      break;;
+			      }
+			   
+			   if (strcmp(item,"ro") == 0)
+			      {
+			      MOUNTMODE='o';
+			      break;
+			      }
+			   }
+      
+                        yyerror("miscmounts: host:/frompath /mounton_path ro|rw\n");
                         break;
 
       case unmounta:    yyerror("Umount must be in the format machine:directory");
@@ -925,6 +941,10 @@ switch (ACTION)
                  HandleOptionalProcessAttribute(wildcard);
                  break;
 
+   case misc_mounts:
+                 HandleOptionalMiscMountsItem(wildcard);
+		 break;
+		 
    case shellcommands:
                  HandleOptionalScriptAttribute(wildcard);
 		 break;
@@ -933,6 +953,10 @@ switch (ACTION)
                  HandleOptionalRequired(wildcard);
 		 break;
 
+   case interfaces:
+                 HandleOptionalInterface(wildcard);
+		 break;
+
    case admit:
                  InstallAuthItem(CURRENTAUTHPATH,wildcard,&VADMIT,&VADMITTOP,CLASSBUFF);
 		 break;
@@ -1025,15 +1049,15 @@ else                                    /* parent */
    {
    if (wait(&status) != pid)
       {
-      printf("%s: Wait for child failed\n",VPREFIX);
-      perror("wait");
+      sprintf(OUTPUT,"Wait for child failed\n");
+      CfLog(cfinform,OUTPUT,"wait");
       return false;
       }
    else
       {
       if (WIFSIGNALED(status))
          {
-         Debug("%s: script %s returned: %d\n",VPREFIX,comm,WTERMSIG(status));
+         Debug("Script %s returned: %d\n",comm,WTERMSIG(status));
          return false;
          }
 
@@ -1081,6 +1105,7 @@ InitializeAction()                                   /* Set defaults */
  ACTIONPENDING = false;
  DESTINATION[0] = '\0';
  IMAGEACTION[0] = '\0';
+ strcpy(VIFNAME,"");
  PTRAVLINKS = (short) '?';
  FORCECOPY = false;
  IMAGEBACKUP = true;
@@ -1092,6 +1117,8 @@ InitializeAction()                                   /* Set defaults */
  COPYTYPE = 't';
  LINKDIRS = 'k';
  USESHELL = 'y';
+ LOGP = 'd';
+ INFORMP = 'd';
  PURGE = 'n';
  TIDYDIRS = false;
  VEXCLUDEPARSE = NULL;
@@ -1103,6 +1130,16 @@ InitializeAction()                                   /* Set defaults */
  strcpy(CFSERVER,"localhost");
  DISCOMP='>';
  DISABLESIZE=cfnosize;
+
+ if (MOUNT_FROM && MOUNT_ONTO)
+    {
+    Debug("Resetting miscmount data\n");
+    MOUNT_FROM = false;
+    MOUNT_ONTO = false;
+    MOUNTMODE='w';
+    MOUNTFROM[0] = '\0';
+    MOUNTONTO[0] = '\0';
+    }
  
  /* Make sure we don't clean the buffer in the middle of a link! */
 
