@@ -161,6 +161,92 @@ return NULL;
 
 #endif
 
+/***********************************************************/
+/* strdup() missing on old BSD systems                     */
+/***********************************************************/
+
+#ifndef HAVE_STRDUP
+
+char *strdup(str)
+
+char *str;
+
+{ char *sp;
+ 
+if (str == NULL)
+   {
+   return NULL;
+   }
+
+if ((sp = malloc(strlen(str)+1)) == NULL)
+   {
+   perror("malloc");
+   return NULL;
+   }
+
+strcpy(sp,str);
+return sp; 
+}
+
+#endif
+
+
+
+/***********************************************************/
+/* strrchr() missing on old BSD systems                     */
+/***********************************************************/
+
+#ifndef HAVE_STRRCHR
+
+char *strrchr(str,ch)
+
+char *str;
+char ch;
+
+{ char *sp;
+ 
+if (str == NULL)
+   {
+   return NULL;
+   }
+
+if (strlen(str) == 0)
+   {
+   return NULL;
+   }
+
+for (sp = str+strlen(str)-1; sp > str; sp--)
+   {
+   if (*sp == ch)
+      {
+      return *sp;
+      }
+   }
+
+return NULL; 
+}
+
+#endif
+
+
+/***********************************************************/
+/* strerror() missing on systems                           */
+/***********************************************************/
+
+#ifndef HAVE_STRERROR
+
+char *strerror(err)
+
+int err;
+
+{ static char buffer[20];
+
+sprintf(buffer,"Error number %d\n",err);
+return buffer; 
+}
+
+#endif
+
 /***********************************************************/
 /* putenv() missing on old BSD systems                     */
 /***********************************************************/
@@ -177,8 +263,7 @@ putenv (s)
 char *s;
 
 {
-Verbose("cfengine: This system does not have putenv()\n");
-Verbose("          Cannot update CFALLCLASSES\n");
+Verbose("(This system does not have putenv: cannot update CFALLCLASSES\n");
 }
 
 
