@@ -75,12 +75,12 @@ DoProcessCheck(pp,procdata)
 struct Process *pp;
 struct Item *procdata;
 
-{ struct Item *ip;
+{ struct Item *ip, *ip2;
   char fastmap[(1 << BYTEWIDTH)];
   struct re_pattern_buffer buf;
   struct re_registers regs;
   char *err, line[bufsize], *sp;
-  int match, pid=-1, ret, matches=0;
+  int match, pid=-1, ret, matches=0, got;
   int BreakPipe();
   FILE *pipe;
 
@@ -107,7 +107,40 @@ for (ip = procdata; ip != NULL; ip=ip->next)
    if ((match = re_search(&buf,ip->name,strlen(ip->name),0,strlen(ip->name),&regs)) >= 0)
       {
       pid = -1;
+      got = true;
 
+      for (ip2 = pp->inclusions; ip2 != NULL; ip2 = ip2->next)
+	 {
+	 got = false;
+	 
+	 if (strstr(ip->name,ip2->name))
+	    {
+	    got = true;
+	    break;
+	    }
+	 }
+   
+      if (!got)
+	 {
+	 continue;
+	 }
+
+      got = false;
+   
+      for (ip2 = pp->exclusions; ip2 != NULL; ip2 = ip2->next)
+	 {
+	 if (strstr(ip->name,ip2->name))
+	    {
+	    got = true;
+	    break;
+	    }
+	 }
+
+      if (got)
+	 {
+	 continue;
+	 }
+      
       for (sp = ip->name; *sp != '\0'; sp++) /* if first field contains alpha, skip */
 	 {
 	 while (!isdigit(*sp) && (*sp != '\0'))
@@ -246,11 +279,15 @@ if (strlen(pp->restart) != 0)
       }
 
    DEADLOCK = false;   
-   signal(SIGALRM,(void *)BreakPipe);
-   alarm(30);
    
    while (!feof(PIPE))
       {
+      if (pp->useshell == 'd') /* dumb shell */
+	 {
+	 fgets(line,1,PIPE);
+         break;
+	 }
+      
       ReadLine(line,bufsize,PIPE);
 
       if (feof(PIPE) || ferror(PIPE))
@@ -275,9 +312,6 @@ if (strlen(pp->restart) != 0)
       CfLog(cfinform,OUTPUT,"");
       }
 
-   alarm(0);
-   signal(SIGALRM,SIG_DFL);
-
    if (pp->useshell == 'y')
       {
       pclose(PIPE);
@@ -292,13 +326,3 @@ if (strlen(pp->restart) != 0)
    }
 }
 
-/*******************************************************************/
-
-BreakPipe()
-
-{
-Verbose("%s: Workaroud on ERESTARTSYS popen() zombie deadlock bug\n",VPREFIX);
-ungetc('\n',PIPE);
-ungetc(EOF,PIPE);
-DEADLOCK = true;
-}
