@@ -37,6 +37,7 @@ char *VVNAMES[] =
    "site",
    "host",
    "fqhost",
+   "ipaddress",
    "binserver",
    "sysadm",
    "domain",
@@ -50,6 +51,7 @@ char *VVNAMES[] =
    "mountpattern",
    "homepattern",
    "addclasses",
+   "addinstallable",
    "access",
    "class",
    "arch",
@@ -70,7 +72,7 @@ char *VVNAMES[] =
    "quote",
    "dollar",
    "repchar",
-   "Split",
+   "split",
    "underscoreclasses",
    "interfacename",
    "expireafter",
@@ -271,8 +273,15 @@ for (sp = string; /* No exit */ ; sp++)       /* check for varitems */
 		      }
                     strcat(buffer,VFQNAME);
                     break;
-	     
 
+	 case cfipaddr:
+	           if (BufferOverflow(buffer,VIPADDRESS))
+		      {
+		      FatalError("Can't expand varstring");
+		      }
+		   strcat(buffer,VIPADDRESS);
+		   break;
+		   
          case cfbinserver:
                     if (ACTION != links && ACTION != required)
                        {
@@ -310,7 +319,7 @@ for (sp = string; /* No exit */ ; sp++)       /* check for varitems */
 		      {
 		      FatalError("Can't expandvarstring");
 		      }
-                   strcat(buffer,VDOMAIN);
+                   strcat(buffer,ToLowerStr(VDOMAIN));
                    break;
 
          case cfnfstype:
@@ -322,16 +331,16 @@ for (sp = string; /* No exit */ ; sp++)       /* check for varitems */
                    break;
 
          case cftimezone:
-                   if (VTIMEZONE[0] == '\0')
+                   if (VTIMEZONE == NULL)
                       {
                       yyerror("timezone undefined variable");
                       }
 
-		   if (BufferOverflow(buffer,VTIMEZONE))
+		   if (BufferOverflow(buffer,VTIMEZONE->name))
 		      {
 		      FatalError("Can't expandvarstring");
 		      }
-                   strcat(buffer,VTIMEZONE);
+                   strcat(buffer,VTIMEZONE->name);
                    break;
 
          case cfclass:
